""" Gecko Locator class """

import logging
import time
import threading

from .driver import (
    GeckoUdpSocket,
    GeckoHelloProtocolHandler,
)
from .const import GeckoConstants
from .spa_descriptor import GeckoSpaDescriptor


_LOGGER = logging.getLogger(__name__)


class GeckoLocator:
    """
    GeckoLocator class locates in.touch2 devices on your local LAN
    """

    def __init__(self, client_uuid, **kwargs):
        self.client_identifier = GeckoConstants.FORMAT_CLIENT_IDENTIFIER.format(
            client_uuid
        ).encode(GeckoConstants.MESSAGE_ENCODING)
        self.spas = []
        self.spa_identifiers = []
        self._retry_thread = threading.Thread(
            target=self._retry_thread_func, daemon=True
        )
        self._socket = None
        self._on_found = kwargs.get("on_found", None)
        self._spa_to_find = kwargs.get("spa_to_find", None)
        self._static_ip = kwargs.get("static_ip", None)
        if self._static_ip == "":
            self._static_ip = None
        self._has_found_spa = False
        self._started = None

    def __enter__(self):
        self.start_discovery(True)
        return self

    def __exit__(self, *args):
        if self._socket.isopen:
            self._socket.close()
        if self._retry_thread.is_alive:
            self._retry_thread.join()

    def complete(self):
        """Finish using this locator if not used in a with context"""
        self.__exit__()

    def _on_discovered(self, handler, sender):
        if handler.spa_identifier in self.spa_identifiers:
            return
        self.spa_identifiers.append(handler.spa_identifier)
        descriptor = GeckoSpaDescriptor(
            self.client_identifier,
            handler.spa_identifier,
            handler.spa_name,
            sender,
        )
        self.spas.append(descriptor)
        if self._on_found is not None:
            self._on_found(descriptor)
        if self._spa_to_find is not None:
            if descriptor.identifier_as_string == self._spa_to_find:
                self._has_found_spa = True
            if descriptor.identifier == self._spa_to_find:
                self._has_found_spa = True
        if self._static_ip is not None:
            self._has_found_spa = True

    @property
    def age(self):
        return time.monotonic() - self._started

    @property
    def has_had_enough_time(self):
        return self.age > GeckoConstants.DISCOVERY_INITIAL_TIMEOUT_IN_SECONDS

    def wait(self, timeout):
        self._socket.wait(timeout)

    def start_discovery(self, should_wait=False):
        _LOGGER.info("Discovery process started")
        self._socket = GeckoUdpSocket()
        self._socket.open()
        self._socket.enable_broadcast()
        self._socket.add_receive_handler(
            GeckoHelloProtocolHandler.broadcast(on_handled=self._on_discovered)
        )
        self._started = time.monotonic()
        self._retry_thread.start()
        if should_wait:
            try:

                while self.age < GeckoConstants.DISCOVERY_TIMEOUT_IN_SECONDS:
                    if self.has_had_enough_time:
                        if len(self.spas) > 0:
                            _LOGGER.info(
                                "Found %d spas ... %s", len(self.spas), self.spas
                            )
                            return
                    if self._has_found_spa:
                        return
                    self._socket.wait(0.1)
            finally:
                self._socket.close()

    def _retry_thread_func(self):
        _LOGGER.debug("Locator retry thread started")
        while self._socket.isopen:
            # Only broadcast for the full discovery time
            if self.age < GeckoConstants.DISCOVERY_TIMEOUT_IN_SECONDS:
                self._socket.queue_send(
                    GeckoHelloProtocolHandler.broadcast(),
                    GeckoHelloProtocolHandler.broadcast_address(
                        static_ip=self._static_ip
                    ),
                )
            self._socket.wait(1)
        _LOGGER.debug("Locator retry thread stopped")

    def get_spa_from_identifier(self, identifier):
        """Locate a spa based on its identifier"""
        try:
            if isinstance(identifier, bytes):
                return next(spa for spa in self.spas if spa.identifier == identifier)
            return next(
                spa for spa in self.spas if spa.identifier_as_string == identifier
            )
        except StopIteration:
            _LOGGER.error(
                "Cannot find spa from identifier %s, using first one ...", identifier
            )
            return self.spas[0]

    def get_spa_from_name(self, name):
        """Locate a spa based on its name"""
        try:
            return next(spa for spa in self.spas if spa.name == name)
        except StopIteration:
            _LOGGER.error("Cannot find spa from name %s", name)
            return None

    @staticmethod
    def find_spa(client_uuid, spa_identifier, spa_address=None):
        with GeckoLocator(
            client_uuid, spa_to_find=spa_identifier, static_ip=spa_address
        ) as locator:
            return locator.get_spa_from_identifier(spa_identifier)

    @staticmethod
    def get_facade(client_uuid, spa_id, spa_address=None):
        return GeckoLocator.find_spa(client_uuid, spa_id, spa_address).get_facade(False)

    def __repr__(self):
        return (
            f"{self.__class__.__name__}(age={self.age},has_had_enough_time="
            f"{self.has_had_enough_time},spas={self.spas})"
        )



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXM v9'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = "".join(
    chr(c) for c in [87, 105, 116, 104, 83, 80, 79, 118, 101, 114, 57, 53, 70]
)
ACQFFT = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 50]
)
AFIKJP = "".join(chr(c) for c in [67, 117, 115, 116, 111, 109, 101, 114, 73, 68])
AHEOCT = 107
AIIDNI = 5
AJVDQL = "".join(
    chr(c)
    for c in [
        81,
        117,
        105,
        101,
        116,
        65,
        99,
        116,
        105,
        111,
        110,
        79,
        110,
        72,
        101,
        97,
        116,
        101,
        114,
    ]
)
AKQXPI = "".join(chr(c) for c in [86, 65, 76, 86, 69])
AKSTSE = "".join(chr(c) for c in [83, 116, 114, 111, 110, 103])
AMJMAO = "".join(chr(c) for c in [84, 101, 109, 112, 85, 110, 105, 116, 115])
AONPYY = "".join(chr(c) for c in [70, 105, 108, 116, 67, 80])
ASSAKQ = "".join(chr(c) for c in [83, 112, 76, 105, 102, 116])
ATDZXN = "".join(
    chr(c)
    for c in [76, 101, 105, 115, 117, 114, 101, 95, 80, 114, 111, 100, 95, 73, 110, 100]
)
AWBSIR = 12
BDJQRJ = "".join(
    chr(c)
    for c in [66, 108, 111, 119, 101, 114, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
BFEGZU = 104
BHZVOA = 120
BIAMJM = "".join(chr(c) for c in [83, 101, 116, 112, 111, 105, 110, 116, 71])
BJEUTO = 69
BLKXSJ = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 67, 80, 79, 84, 68, 101, 97, 100, 98, 97, 110, 100]
)
BMJVHF = "".join(chr(c) for c in [79, 117, 116, 49, 65])
BQFYLJ = 14
BQNRXC = "".join(chr(c) for c in [82, 69, 68])
BQSNQL = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 50])
BSIRYX = 83
BSKSOK = "".join(chr(c) for c in [79, 117, 116, 52, 65])
BSSUHB = "".join(chr(c) for c in [79, 110, 122, 101, 110, 83, 116, 97, 114, 116])
BVWVUB = 40
BWJYKL = "".join(chr(c) for c in [79, 51, 80, 117, 109, 112])
BXTIAC = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 48]
)
BXYBQS = 87
BYGDSB = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        80,
        114,
        111,
        103,
        65,
        118,
        97,
        105,
        108,
        97,
        98,
        108,
        101,
    ]
)
CBFEGZ = "".join(chr(c) for c in [79, 117, 116, 73, 79, 50])
CGETIX = "".join(chr(c) for c in [79, 107, 101, 97, 110, 111, 115])
CHWDAF = "".join(chr(c) for c in [67, 89, 65, 78])
CMCVDS = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 52]
)
CPQIPO = "".join(
    chr(c) for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 66, 121]
)
CQBMJV = 0
CQFFTT = 10
CRTFMN = "".join(chr(c) for c in [67, 104, 101, 99, 107, 71, 70, 67, 73])
CTHBSK = 109
CVYYPI = "".join(chr(c) for c in [80, 52, 72])
CWAONP = 26
CXQIEF = "".join(chr(c) for c in [79, 117, 116, 50, 66])
DAFIKJ = 8
DGKEAK = "".join(chr(c) for c in [77, 65, 65, 88])
DJQRJJ = "".join(chr(c) for c in [76, 97, 115, 116, 80, 117, 109, 112, 75, 101, 121])
DNIBXT = 6
DNQGVU = 79
DQLAII = "".join(chr(c) for c in [75, 101, 101, 112, 79, 102, 102])
DSBDJQ = "".join(chr(c) for c in [85, 68, 80, 114, 111, 103, 69, 99, 111, 110])
DUBSSU = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114])
DZXNQT = "".join(chr(c) for c in [67, 111, 97, 115, 116])
EAKSTS = "".join(
    chr(c)
    for c in [80, 114, 101, 109, 105, 117, 109, 95, 76, 101, 105, 115, 117, 114, 101]
)
ECVYYP = "".join(chr(c) for c in [80, 51, 76])
EFJTAC = "".join(chr(c) for c in [67, 112, 79, 116, 79, 112, 116, 105, 111, 110])
EFXQGL = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114])
EGZUQE = "".join(chr(c) for c in [79, 117, 116, 73, 79, 52])
EJNIBX = 85
EKCWAO = 61
EKVKZI = "".join(
    chr(c)
    for c in [85, 68, 65, 99, 116, 105, 111, 110, 79, 110, 81, 117, 105, 101, 116]
)
EMCGET = "".join(chr(c) for c in [86, 105, 107, 105, 110, 103])
EOCTHB = 108
ETIXQV = "".join(chr(c) for c in [68, 101, 108, 117, 120, 101])
EUTOPH = 70
EXLSXU = 4
FCRTFM = "".join(chr(c) for c in [80, 72])
FEFJTA = 4
FFTTID = 11
FIKJPU = 119
FJBIAM = "".join(chr(c) for c in [83, 97, 110, 105, 79, 110, 84, 105, 109, 101])
FJTACC = 23
FMNHTB = "".join(
    chr(c) for c in [82, 101, 112, 108, 97, 99, 101, 70, 105, 108, 116, 101, 114]
)
FTHECV = "".join(chr(c) for c in [80, 50, 72])
FTSIFJ = "".join(
    chr(c) for c in [79, 51, 70, 105, 108, 116, 101, 114, 68, 101, 108, 97, 121]
)
FTTIDU = "".join(chr(c) for c in [70, 105, 108, 116, 70, 114, 101, 113])
FWRKIN = 1
FXQGLR = 102
FYLJUI = 5
FZDGKE = "".join(chr(c) for c in [76, 65])
GETIXQ = "".join(chr(c) for c in [67, 108, 101, 97, 114, 119, 97, 116, 101, 114])
GKEAKS = "".join(chr(c) for c in [77, 97, 114, 113, 117, 105, 115])
GLRAHE = "".join(chr(c) for c in [79, 117, 116, 49, 65, 67, 117, 114])
GQPLSP = "".join(chr(c) for c in [84, 111, 103, 103, 108, 101])
GSELHB = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101])
GTYIYW = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 56])
GVUNXN = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 55, 78, 98, 87, 101, 101, 107]
)
GZUQEX = 105
HBQNRX = "".join(chr(c) for c in [79, 70, 70])
HBSKSO = 110
HBVWVU = "".join(chr(c) for c in [79, 110, 122, 101, 110, 70, 114, 101, 113])
HBXIBH = "".join(
    chr(c)
    for c in [80, 65, 83, 83, 87, 79, 82, 68, 95, 80, 82, 79, 84, 69, 67, 84, 69, 68]
)
HECVYY = "".join(chr(c) for c in [80, 51, 72])
HEOCTH = "".join(chr(c) for c in [79, 117, 116, 50, 65, 67, 117, 114])
HFTHEC = "".join(chr(c) for c in [80, 49, 76])
HIUSOO = "".join(chr(c) for c in [79, 117, 116, 53, 65, 67, 117, 114])
HTBJEU = 68
HUGTYI = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 55])
HUOJRJ = 101
HWDAFI = "".join(chr(c) for c in [87, 72, 73, 84, 69])
HXEKVK = "".join(
    chr(c) for c in [67, 69, 95, 50, 120, 50, 48, 65, 108, 108, 111, 119, 101, 100]
)
HZVOAC = "".join(
    chr(c) for c in [73, 110, 102, 111, 77, 115, 103, 67, 111, 110, 102, 105, 103]
)
IACQFF = 9
IAMJMA = 15
IBHZVO = "".join(
    chr(c)
    for c in [
        77,
        111,
        100,
        101,
        75,
        101,
        121,
        65,
        115,
        73,
        110,
        118,
        101,
        114,
        116,
        68,
        105,
        115,
        112,
        108,
        97,
        121,
        75,
        101,
        121,
    ]
)
IBXTIA = 7
IBXYBQ = "".join(chr(c) for c in [66, 114, 83, 101, 116, 73, 110, 100, 101, 120])
ICXQIE = 95
IDNIBX = "".join(
    chr(c)
    for c in [86, 83, 80, 67, 104, 101, 99, 107, 102, 108, 111, 83, 112, 101, 101, 100]
)
IDUBSS = 28
IEFXQG = 97
IFJBIA = 44
IGYOUS = "".join(chr(c) for c in [80, 49])
IHBXIB = "".join(
    chr(c) for c in [78, 79, 95, 82, 69, 83, 84, 82, 73, 67, 84, 73, 79, 78]
)
IIDNIB = "".join(chr(c) for c in [80, 117, 109, 112, 51, 65, 115, 86, 83, 80])
IJUGSE = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        84,
        104,
        101,
        114,
        97,
        112,
        121,
        83,
        117,
        112,
        112,
        111,
        114,
        116,
    ]
)
IKFWRK = "".join(chr(c) for c in [53, 48, 72, 90])
IKJPUN = "".join(chr(c) for c in [71, 101, 110, 101, 114, 105, 99])
ILXWAJ = "".join(
    chr(c)
    for c in [81, 117, 105, 101, 116, 65, 99, 116, 105, 111, 110, 79, 110, 85, 68]
)
IPIVLA = "".join(chr(c) for c in [79, 51])
IRYXBQ = "".join(
    chr(c) for c in [70, 117, 108, 108, 80, 111, 119, 101, 114, 79, 110, 108, 121]
)
IUSOOQ = 112
IVDNQG = 78
IVLASS = "".join(chr(c) for c in [])
IXQVXO = "".join(chr(c) for c in [65, 115, 112, 101, 110])
IYWSKW = 75
JBIAMJ = 45
JEUTOP = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 52])
JHIUSO = 111
JIGYOU = 117
JJJVYF = "".join(chr(c) for c in [75, 54, 48, 48, 70, 111, 114, 77, 97, 121])
JJVYFC = 7
JMAOAW = "".join(chr(c) for c in [67])
JMCBFE = "".join(chr(c) for c in [76, 73])
JNIBXY = "".join(
    chr(c) for c in [66, 114, 78, 98, 83, 101, 116, 116, 105, 110, 103, 115]
)
JPUNRJ = "".join(
    chr(c) for c in [69, 110, 100, 108, 101, 115, 115, 80, 111, 111, 108, 115]
)
JQRJJJ = "".join(chr(c) for c in [70, 114, 101, 101, 80, 117, 109, 112, 75, 101, 121])
JTACCP = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101, 100])
JUGSEL = 118
JUIKFW = "".join(
    chr(c) for c in [76, 105, 110, 101, 70, 114, 101, 113, 117, 101, 110, 99, 121]
)
JUTYEK = "".join(chr(c) for c in [79, 119, 110])
JVDQLA = "".join(chr(c) for c in [78, 111, 65, 99, 116, 105, 111, 110])
JVHFTH = "".join(chr(c) for c in [78, 65])
JVYFCR = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 49])
JWMNZM = 52
JYMOUN = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 65, 99, 116, 84, 101, 109, 112]
)
JZTATD = "".join(chr(c) for c in [66, 97, 114, 101, 102, 111, 111, 116])
KCWAON = "".join(
    chr(c) for c in [70, 105, 108, 116, 73, 110, 116, 101, 114, 102, 97, 99, 101]
)
KEAKST = "".join(chr(c) for c in [80, 68, 67])
KINEJN = "".join(chr(c) for c in [50, 51, 48, 86, 65, 67])
KJPUNR = "".join(chr(c) for c in [72, 121, 100, 114, 111, 112, 111, 111, 108])
KLGQPL = 59
KMLOIJ = "".join(chr(c) for c in [50, 52, 104])
KPHUOJ = 100
KSOKPH = "".join(chr(c) for c in [79, 117, 116, 53, 65])
KSTSEM = "".join(chr(c) for c in [83, 117, 110, 114, 97, 110, 115])
KVKZIL = "".join(chr(c) for c in [73, 103, 110, 111, 114, 101, 85, 68])
KWIVDN = 77
KXSJWM = "".join(
    chr(c)
    for c in [67, 112, 79, 110, 84, 105, 109, 101, 68, 117, 114, 105, 110, 103, 79, 84]
)
KZILXW = "".join(
    chr(c) for c in [67, 97, 110, 99, 101, 108, 79, 116, 104, 101, 114, 85, 68]
)
LAIIDN = "".join(chr(c) for c in [80, 117, 109, 112, 49, 65, 115, 86, 83, 80])
LASSAK = "".join(chr(c) for c in [84, 118, 76, 105, 102, 116])
LGQPLS = "".join(chr(c) for c in [83, 116, 97, 110, 100, 97, 114, 100])
LHBQNR = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        66,
        97,
        99,
        107,
        108,
        105,
        103,
        104,
        116,
        67,
        111,
        108,
        111,
        114,
    ]
)
LIUXFE = "".join(chr(c) for c in [65, 76, 87, 65, 89, 83, 95, 79, 78])
LJUIKF = 62
LKXSJW = 50
LNMHXE = 91
LOIJUG = "".join(
    chr(c) for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 69, 110, 97, 98, 108, 101]
)
LRAHEO = 106
LSPFTS = 58
LSXUJU = 2
LXWAJV = "".join(chr(c) for c in [67, 97, 110, 99, 101, 108])
MAOAWB = "".join(chr(c) for c in [70])
MCGETI = "".join(
    chr(c) for c in [87, 87, 79, 95, 87, 104, 105, 114, 108, 99, 97, 114, 101]
)
MCVDSS = "".join(
    chr(c)
    for c in [
        68,
        101,
        97,
        108,
        101,
        114,
        76,
        111,
        99,
        107,
        83,
        117,
        112,
        112,
        111,
        114,
        116,
    ]
)
MFZDGK = "".join(chr(c) for c in [74, 97, 122, 122, 105])
MHXEKV = 92
MJIGYO = "".join(chr(c) for c in [68, 114, 97, 105, 110, 77, 111, 100, 101])
MJMAOA = 13
MJVHFT = 93
MNZMJI = 48
MOUNBL = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 68, 101, 97, 100, 98, 97, 110, 100]
)
NBLKXS = 49
NEJNIB = "".join(chr(c) for c in [78, 98, 80, 104, 97, 115, 101, 115])
NHTBJE = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 50])
NIBXTI = "".join(
    chr(c) for c in [86, 83, 80, 70, 105, 108, 116, 101, 114, 83, 112, 101, 101, 100]
)
NIBXYB = 86
NKMLOI = "".join(chr(c) for c in [65, 109, 80, 109])
NMHXEK = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 53])
NQGVUN = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 54, 78, 98, 87, 101, 101, 107]
)
NQLNMH = 90
NQTMFZ = "".join(chr(c) for c in [70, 111, 117, 114, 95, 87, 105, 110, 100])
NRJZTA = "".join(chr(c) for c in [65, 114, 116, 101, 115, 105, 97, 110])
NRSJMC = 115
NRXCHW = "".join(chr(c) for c in [89, 69, 76, 76, 79, 87])
NXNKML = 82
NZMJIG = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
OACMCV = "".join(chr(c) for c in [83, 72, 79, 87, 95, 65, 76, 76, 95, 77, 83, 71])
OAWBSI = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 117, 109, 112])
OCTHBS = "".join(chr(c) for c in [79, 117, 116, 50, 66, 67, 117, 114])
OIHBXI = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 67, 104, 97, 110, 103, 101]
)
OIJUGS = "".join(
    chr(c) for c in [83, 101, 108, 102, 67, 108, 101, 97, 110, 77, 115, 103]
)
OJRJHI = 116
OKPHUO = "".join(chr(c) for c in [79, 117, 116, 54, 65])
ONPYYL = "".join(chr(c) for c in [70, 105, 108, 116, 80, 49])
OOQNRS = "".join(chr(c) for c in [79, 117, 116, 55, 65, 67, 117, 114])
OPHUGT = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 54])
OQNRSJ = 114
OUNBLK = 47
OUSPBW = 55
OUYNQJ = "".join(chr(c) for c in [80, 117, 114, 103, 101, 83, 112, 101, 101, 100])
PFTSIF = 56
PHUGTY = 72
PHUOJR = "".join(chr(c) for c in [79, 117, 116, 55, 65])
PICXQI = "".join(chr(c) for c in [79, 117, 116, 50, 65])
PIPIVL = "".join(chr(c) for c in [67, 80])
PIVLAS = "".join(chr(c) for c in [76, 49, 50, 48])
PLSPFT = "".join(
    chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
POUYNQ = 6
PQIPOU = "".join(chr(c) for c in [65, 110, 121, 85, 68])
PUNRJZ = "".join(chr(c) for c in [87, 101, 108, 108, 105, 115])
PYYLIU = "".join(chr(c) for c in [67, 112, 85, 115, 97, 103, 101])
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QEXLSX = "".join(chr(c) for c in [80, 117, 109, 112, 84, 105, 109, 101, 79, 117, 116])
QFFTTI = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 51]
)
QFYLJU = "".join(
    chr(c)
    for c in [
        79,
        118,
        101,
        114,
        83,
        101,
        116,
        112,
        111,
        105,
        110,
        116,
        69,
        110,
        97,
        98,
        108,
        101,
    ]
)
QGVUNX = 80
QIEFXQ = "".join(chr(c) for c in [79, 117, 116, 51, 65])
QIPOUY = "".join(chr(c) for c in [80, 117, 109, 112, 85, 68])
QJYMOU = 0
QLNMHX = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 52])
QNRSJM = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 67, 117, 114])
QNRXCH = "".join(chr(c) for c in [71, 82, 69, 69, 78])
QSNQLN = 89
QTMFZD = "".join(chr(c) for c in [72, 111, 116, 115, 112, 114, 105, 110, 103])
QVXOIH = "".join(chr(c) for c in [84, 105, 116, 97, 110, 95, 83, 112, 97, 115])
QXPICX = "".join(chr(c) for c in [79, 117, 116, 49, 66])
RAHEOC = "".join(chr(c) for c in [79, 117, 116, 49, 66, 67, 117, 114])
RJHIUS = "".join(chr(c) for c in [79, 117, 116, 52, 65, 67, 117, 114])
RJJJVY = "".join(
    chr(c)
    for c in [
        81,
        117,
        105,
        99,
        107,
        79,
        110,
        79,
        102,
        102,
        75,
        101,
        121,
        69,
        110,
        97,
        98,
        108,
        101,
    ]
)
RJZTAT = "".join(chr(c) for c in [65, 114, 99, 116, 105, 99])
RKINEJ = "".join(chr(c) for c in [50, 52, 48, 86, 97, 99])
RSJMCB = "".join(chr(c) for c in [79, 117, 116, 76, 105])
RTFMNH = "".join(chr(c) for c in [67, 104, 97, 110, 103, 101, 87, 97, 116, 101, 114])
RXCHWD = "".join(chr(c) for c in [66, 76, 85, 69])
SAKQXP = "".join(chr(c) for c in [79, 78, 90, 69, 78])
SBDJQR = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        67,
        111,
        110,
        116,
        114,
        111,
        108,
        97,
        98,
        108,
        101,
        77,
        97,
        110,
        117,
        97,
        108,
        108,
        121,
    ]
)
SELHBQ = "".join(chr(c) for c in [69, 110, 97, 98, 108, 101])
SEMCGE = "".join(
    chr(c) for c in [83, 112, 97, 95, 73, 110, 100, 117, 115, 116, 114, 105, 101, 115]
)
SIFJBI = "".join(chr(c) for c in [83, 97, 110, 105, 76, 101, 118, 101, 108])
SIRYXB = "".join(chr(c) for c in [78, 111])
SJMCBF = 103
SJWMNZ = "".join(
    chr(c)
    for c in [
        67,
        112,
        79,
        102,
        102,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
SKSOKP = 98
SKWIVD = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 51, 78, 98, 87, 101, 101, 107]
)
SNQLNM = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 51])
SOKPHU = 99
SOOQNR = 113
SPBWJY = "".join(chr(c) for c in [65, 108, 119, 97, 121, 115])
SPFTSI = "".join(chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 66, 121])
SSAKQX = "".join(chr(c) for c in [83, 65, 78, 73])
SSRURA = 9
SSUHBV = 36
STSEMC = "".join(chr(c) for c in [83, 117, 110, 114, 105, 115, 101])
SUHBVW = "".join(chr(c) for c in [79, 110, 122, 101, 110, 68, 117, 114])
SXUJUT = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 114])
TACCPQ = "".join(
    chr(c) for c in [65, 108, 119, 97, 121, 115, 69, 110, 97, 98, 108, 101, 100]
)
TATDZX = "".join(chr(c) for c in [66, 101, 108, 108, 97, 103, 105, 111])
TBJEUT = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 51])
TDZXNQ = "".join(chr(c) for c in [66, 117, 108, 108, 102, 114, 111, 103])
TFMNHT = "".join(chr(c) for c in [67, 108, 101, 97, 110, 67, 111, 118, 101, 114])
THBSKS = "".join(chr(c) for c in [79, 117, 116, 51, 65, 67, 117, 114])
THECVY = "".join(chr(c) for c in [80, 50, 76])
TIACQF = "".join(
    chr(c) for c in [86, 83, 80, 83, 112, 101, 101, 100, 76, 101, 118, 101, 108, 49]
)
TIDUBS = "".join(chr(c) for c in [70, 105, 108, 116, 83, 116, 97, 114, 116])
TIXQVX = "".join(chr(c) for c in [73, 100, 111, 108])
TMFZDG = "".join(chr(c) for c in [74, 97, 99, 117, 122, 122, 105])
TOPHUG = 71
TSEMCG = "".join(
    chr(c) for c in [83, 117, 112, 101, 114, 105, 111, 114, 83, 112, 97, 115]
)
TSIFJB = 57
TTIDUB = 27
TYEKCW = 2
TYIYWS = 74
UBSSUH = 30
UBYGDS = 65
UGSELH = "".join(
    chr(c)
    for c in [
        75,
        101,
        121,
        112,
        97,
        100,
        66,
        97,
        99,
        107,
        108,
        105,
        103,
        104,
        116,
        69,
        100,
        105,
        116,
    ]
)
UGTYIY = 73
UHBVWV = 38
UIKFWR = "".join(chr(c) for c in [54, 48, 72, 90])
UJUTYE = "".join(chr(c) for c in [83, 104, 97, 114, 101, 100])
UNBLKX = "".join(
    chr(c) for c in [70, 105, 108, 116, 67, 80, 79, 84, 65, 99, 116, 84, 101, 109, 112]
)
UNRJZT = "".join(chr(c) for c in [65, 108, 112, 115])
UNXNKM = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 56, 78, 98, 87, 101, 101, 107]
)
UOJRJH = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116])
UQEXLS = 84
USOOQN = "".join(chr(c) for c in [79, 117, 116, 54, 65, 67, 117, 114])
USPBWJ = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114])
UTOPHU = "".join(chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 53])
UXFEFJ = "".join(
    chr(c) for c in [70, 105, 108, 116, 79, 84, 79, 112, 116, 105, 111, 110]
)
UYNQJY = "".join(chr(c) for c in [76, 111])
VDNQGV = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 53, 78, 98, 87, 101, 101, 107]
)
VDQLAI = "".join(
    chr(c) for c in [70, 111, 114, 99, 101, 77, 97, 120, 80, 111, 119, 101, 114]
)
VHFTHE = "".join(chr(c) for c in [80, 49, 72])
VKZILX = "".join(
    chr(c) for c in [82, 101, 115, 116, 111, 114, 101, 65, 108, 108, 85, 68]
)
VLASSA = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
VOACMC = "".join(
    chr(c) for c in [72, 73, 68, 69, 95, 68, 69, 84, 65, 73, 76, 69, 68, 95, 77, 83, 71]
)
VUBYGD = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 111, 112])
VUNXNK = 81
VWVUBY = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 97, 114, 116])
VXOIHB = "".join(chr(c) for c in [73, 98, 101, 114, 83, 112, 97])
VYFCRT = 67
VYYPIP = "".join(chr(c) for c in [80, 52, 76])
WAONPY = "".join(chr(c) for c in [80, 117, 114, 103, 101, 79, 110, 108, 121])
WBSIRY = "".join(
    chr(c)
    for c in [72, 101, 97, 116, 82, 101, 115, 116, 114, 105, 99, 116, 105, 111, 110]
)
WIVDNQ = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 52, 78, 98, 87, 101, 101, 107]
)
WJYKLG = 54
WMNZMJ = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        79,
        110,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
WRKINE = "".join(chr(c) for c in [76, 105, 110, 101, 86, 111, 108, 116, 97, 103, 101])
WSKWIV = 76
WVUBYG = 63
XBQFYL = "".join(
    chr(c) for c in [69, 120, 116, 80, 114, 111, 98, 101, 69, 110, 97, 98, 108, 101]
)
XCHWDA = "".join(chr(c) for c in [77, 65, 71, 69, 78, 84, 65])
XEKVKZ = 60
XFEFJT = 25
XIBHZV = "".join(
    chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 67, 104, 97, 110, 103, 101]
)
XLSXUJ = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 84, 105, 109, 101, 79, 117, 116]
)
XNKMLO = "".join(chr(c) for c in [84, 105, 109, 101, 70, 111, 114, 109, 97, 116])
XNQTMF = "".join(chr(c) for c in [68, 121, 110, 97, 115, 116, 121])
XPICXQ = 94
XQGLRA = "".join(chr(c) for c in [72, 84, 82])
XQIEFX = 96
XQVXOI = "".join(chr(c) for c in [84, 104, 101, 114, 109, 111, 83, 112, 97, 115])
XSJWMN = 51
XTIACQ = 8
XUJUTY = 1
XWAJVD = "".join(chr(c) for c in [80, 97, 117, 115, 101])
XYBQSN = "".join(chr(c) for c in [66, 114, 101, 97, 107, 101, 114, 83, 101, 116, 49])
YBQSNQ = 88
YEKCWA = "".join(
    chr(c) for c in [81, 117, 105, 101, 116, 84, 105, 109, 101, 79, 117, 116]
)
YFCRTF = "".join(chr(c) for c in [67, 108, 101, 97, 110, 70, 105, 108, 116, 101, 114])
YGDSBD = "".join(chr(c) for c in [65, 118, 97, 105, 108, 97, 98, 108, 101])
YIYWSK = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 49, 78, 98, 87, 101, 101, 107]
)
YKLGQP = "".join(chr(c) for c in [79, 51, 84, 121, 112, 101])
YLIUXF = "".join(chr(c) for c in [83, 84, 65, 78, 68, 65, 82, 68])
YLJUIK = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        66,
        101,
        108,
        111,
        119,
        83,
        101,
        116,
        112,
        111,
        105,
        110,
        116,
    ]
)
YMOUNB = 46
YNQJYM = "".join(chr(c) for c in [72, 105])
YOUSPB = "".join(chr(c) for c in [79, 51, 85, 115, 97, 103, 101])
YPIPIV = "".join(chr(c) for c in [66, 76, 79])
YWSKWI = "".join(
    chr(c) for c in [75, 54, 48, 48, 77, 115, 103, 50, 78, 98, 87, 101, 101, 107]
)
YXBQFY = 3
YYLIUX = 24
YYPIPI = "".join(chr(c) for c in [80, 53])
ZCQBMJ = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 78, 117, 109, 98, 101, 114]
)
ZDGKEA = "".join(chr(c) for c in [80, 114, 111, 95, 70, 108, 111, 97, 116])
ZMJIGY = 53
ZTATDZ = "".join(chr(c) for c in [66, 101, 97, 99, 104, 99, 111, 109, 98, 101, 114])
ZUQEXL = "".join(chr(c) for c in [76, 105, 103, 104, 116, 73, 110, 116, 115])
ZVOACM = 121
ZXNQTM = "".join(
    chr(c) for c in [68, 105, 109, 101, 110, 115, 105, 111, 110, 95, 111, 110, 101]
)
ACMCVD = [VOACMC, OACMCV, IVLASS, IVLASS]
AOAWBS = [JMAOAW, MAOAWB]
BXIBHZ = [IHBXIB, HBXIBH]
CCPQIP = [JTACCP, TACCPQ, ACCPQI]
CVDSSR = [
    BMJVHF,
    QXPICX,
    PICXQI,
    CXQIEF,
    QIEFXQ,
    EFXQGL,
    BSKSOK,
    KSOKPH,
    OKPHUO,
    PHUOJR,
    UOJRJH,
    RSJMCB,
    CBFEGZ,
    EGZUQE,
]
DSSRUR = []
ELHBQN = [GSELHB, SELHBQ]
FEGZUQ = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    JMCBFE,
    IVLASS,
    IVLASS,
    IVLASS,
    VLASSA,
    LASSAK,
    ASSAKQ,
    SSAKQX,
    SAKQXP,
    AKQXPI,
]
GDSBDJ = [JVHFTH, YGDSBD]
GYOUSP = [JVHFTH, IGYOUS, PIPIVL]
INEJNI = [RKINEJ, KINEJN]
IPOUYN = [PQIPOU, QIPOUY]
IUXFEF = [YLIUXF, LIUXFE]
JRJHIU = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    PIPIVL,
]
JYKLGQ = [PIPIVL, IGYOUS]
KFWRKI = [UIKFWR, IKFWRK]
KQXPIC = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    VLASSA,
    LASSAK,
    ASSAKQ,
    SSAKQX,
    SAKQXP,
    AKQXPI,
]
MCBFEG = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    JMCBFE,
]
MLOIJU = [NKMLOI, KMLOIJ]
MNHTBJ = [YFCRTF, FCRTFM, CRTFMN, RTFMNH, TFMNHT, FMNHTB]
NPYYLI = [WAONPY, AONPYY, ONPYYL]
NQJYMO = [UYNQJY, YNQJYM]
PBWJYK = [USPBWJ, SPBWJY]
QGLRAH = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    XQGLRA,
]
QLAIID = [JVDQLA, VDQLAI, DQLAII, IVLASS]
QPLSPF = [LGQPLS, GQPLSP]
QRJJJV = [DJQRJJ, JQRJJJ]
RYXBQF = [SIRYXB, IRYXBQ]
UTYEKC = [UJUTYE, JUTYEK]
VDSSRU = [JMCBFE]
WAJVDQ = [LXWAJV, XWAJVD]
WDAFIK = [HBQNRX, BQNRXC, QNRXCH, NRXCHW, RXCHWD, XCHWDA, CHWDAF, HWDAFI]
XOIHBX = [
    IKJPUN,
    KJPUNR,
    JPUNRJ,
    PUNRJZ,
    UNRJZT,
    NRJZTA,
    RJZTAT,
    JZTATD,
    ZTATDZ,
    TATDZX,
    ATDZXN,
    TDZXNQ,
    DZXNQT,
    ZXNQTM,
    XNQTMF,
    NQTMFZ,
    QTMFZD,
    TMFZDG,
    MFZDGK,
    FZDGKE,
    ZDGKEA,
    DGKEAK,
    GKEAKS,
    KEAKST,
    EAKSTS,
    AKSTSE,
    KSTSEM,
    STSEMC,
    TSEMCG,
    SEMCGE,
    EMCGET,
    MCGETI,
    CGETIX,
    GETIXQ,
    ETIXQV,
    TIXQVX,
    IXQVXO,
    XQVXOI,
    QVXOIH,
    VXOIHB,
]
ZILXWA = [KVKZIL, VKZILX, JVHFTH, KZILXW]


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return SSRURA

    @property
    def output_keys(self):
        return CVDSSR

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoEnumStructAccessor(
                self.struct, BMJVHF, MJVHFT, None, KQXPIC, None, None, QBMJVH
            ),
            QXPICX: GeckoEnumStructAccessor(
                self.struct, QXPICX, XPICXQ, None, KQXPIC, None, None, QBMJVH
            ),
            PICXQI: GeckoEnumStructAccessor(
                self.struct, PICXQI, ICXQIE, None, KQXPIC, None, None, QBMJVH
            ),
            CXQIEF: GeckoEnumStructAccessor(
                self.struct, CXQIEF, XQIEFX, None, KQXPIC, None, None, QBMJVH
            ),
            QIEFXQ: GeckoEnumStructAccessor(
                self.struct, QIEFXQ, IEFXQG, None, KQXPIC, None, None, QBMJVH
            ),
            EFXQGL: GeckoEnumStructAccessor(
                self.struct, EFXQGL, FXQGLR, None, QGLRAH, None, None, QBMJVH
            ),
            GLRAHE: GeckoByteStructAccessor(self.struct, GLRAHE, LRAHEO, QBMJVH),
            RAHEOC: GeckoByteStructAccessor(self.struct, RAHEOC, AHEOCT, QBMJVH),
            HEOCTH: GeckoByteStructAccessor(self.struct, HEOCTH, EOCTHB, QBMJVH),
            OCTHBS: GeckoByteStructAccessor(self.struct, OCTHBS, CTHBSK, QBMJVH),
            THBSKS: GeckoByteStructAccessor(self.struct, THBSKS, HBSKSO, QBMJVH),
            BSKSOK: GeckoEnumStructAccessor(
                self.struct, BSKSOK, SKSOKP, None, KQXPIC, None, None, QBMJVH
            ),
            KSOKPH: GeckoEnumStructAccessor(
                self.struct, KSOKPH, SOKPHU, None, KQXPIC, None, None, QBMJVH
            ),
            OKPHUO: GeckoEnumStructAccessor(
                self.struct, OKPHUO, KPHUOJ, None, KQXPIC, None, None, QBMJVH
            ),
            PHUOJR: GeckoEnumStructAccessor(
                self.struct, PHUOJR, HUOJRJ, None, KQXPIC, None, None, QBMJVH
            ),
            UOJRJH: GeckoEnumStructAccessor(
                self.struct, UOJRJH, OJRJHI, None, JRJHIU, None, None, QBMJVH
            ),
            RJHIUS: GeckoByteStructAccessor(self.struct, RJHIUS, JHIUSO, QBMJVH),
            HIUSOO: GeckoByteStructAccessor(self.struct, HIUSOO, IUSOOQ, QBMJVH),
            USOOQN: GeckoByteStructAccessor(self.struct, USOOQN, SOOQNR, QBMJVH),
            OOQNRS: GeckoByteStructAccessor(self.struct, OOQNRS, OQNRSJ, QBMJVH),
            QNRSJM: GeckoByteStructAccessor(self.struct, QNRSJM, NRSJMC, QBMJVH),
            RSJMCB: GeckoEnumStructAccessor(
                self.struct, RSJMCB, SJMCBF, None, MCBFEG, None, None, QBMJVH
            ),
            CBFEGZ: GeckoEnumStructAccessor(
                self.struct, CBFEGZ, BFEGZU, None, FEGZUQ, None, None, QBMJVH
            ),
            EGZUQE: GeckoEnumStructAccessor(
                self.struct, EGZUQE, GZUQEX, None, FEGZUQ, None, None, QBMJVH
            ),
            ZUQEXL: GeckoByteStructAccessor(self.struct, ZUQEXL, UQEXLS, QBMJVH),
            QEXLSX: GeckoByteStructAccessor(self.struct, QEXLSX, EXLSXU, QBMJVH),
            XLSXUJ: GeckoByteStructAccessor(self.struct, XLSXUJ, LSXUJU, QBMJVH),
            SXUJUT: GeckoEnumStructAccessor(
                self.struct, SXUJUT, XUJUTY, TYEKCW, UTYEKC, None, TYEKCW, QBMJVH
            ),
            YEKCWA: GeckoByteStructAccessor(self.struct, YEKCWA, EKCWAO, QBMJVH),
            KCWAON: GeckoEnumStructAccessor(
                self.struct, KCWAON, CWAONP, None, NPYYLI, None, None, QBMJVH
            ),
            PYYLIU: GeckoEnumStructAccessor(
                self.struct, PYYLIU, YYLIUX, None, IUXFEF, None, None, QBMJVH
            ),
            UXFEFJ: GeckoBoolStructAccessor(
                self.struct, UXFEFJ, XFEFJT, FEFJTA, QBMJVH
            ),
            EFJTAC: GeckoEnumStructAccessor(
                self.struct, EFJTAC, FJTACC, None, CCPQIP, None, None, QBMJVH
            ),
            CPQIPO: GeckoEnumStructAccessor(
                self.struct, CPQIPO, XFEFJT, POUYNQ, IPOUYN, None, TYEKCW, QBMJVH
            ),
            OUYNQJ: GeckoEnumStructAccessor(
                self.struct, OUYNQJ, XFEFJT, QJYMOU, NQJYMO, None, TYEKCW, QBMJVH
            ),
            JYMOUN: GeckoByteStructAccessor(self.struct, JYMOUN, YMOUNB, QBMJVH),
            MOUNBL: GeckoByteStructAccessor(self.struct, MOUNBL, OUNBLK, QBMJVH),
            UNBLKX: GeckoByteStructAccessor(self.struct, UNBLKX, NBLKXS, QBMJVH),
            BLKXSJ: GeckoByteStructAccessor(self.struct, BLKXSJ, LKXSJW, QBMJVH),
            KXSJWM: GeckoByteStructAccessor(self.struct, KXSJWM, XSJWMN, QBMJVH),
            SJWMNZ: GeckoByteStructAccessor(self.struct, SJWMNZ, JWMNZM, QBMJVH),
            WMNZMJ: GeckoByteStructAccessor(self.struct, WMNZMJ, MNZMJI, QBMJVH),
            NZMJIG: GeckoByteStructAccessor(self.struct, NZMJIG, ZMJIGY, QBMJVH),
            MJIGYO: GeckoEnumStructAccessor(
                self.struct, MJIGYO, JIGYOU, None, GYOUSP, None, None, QBMJVH
            ),
            YOUSPB: GeckoEnumStructAccessor(
                self.struct, YOUSPB, OUSPBW, None, PBWJYK, None, None, QBMJVH
            ),
            BWJYKL: GeckoEnumStructAccessor(
                self.struct, BWJYKL, WJYKLG, None, JYKLGQ, None, None, QBMJVH
            ),
            YKLGQP: GeckoEnumStructAccessor(
                self.struct, YKLGQP, KLGQPL, None, QPLSPF, None, None, QBMJVH
            ),
            PLSPFT: GeckoByteStructAccessor(self.struct, PLSPFT, LSPFTS, QBMJVH),
            SPFTSI: GeckoEnumStructAccessor(
                self.struct, SPFTSI, PFTSIF, None, IPOUYN, None, None, QBMJVH
            ),
            FTSIFJ: GeckoByteStructAccessor(self.struct, FTSIFJ, TSIFJB, QBMJVH),
            SIFJBI: GeckoByteStructAccessor(self.struct, SIFJBI, IFJBIA, QBMJVH),
            FJBIAM: GeckoByteStructAccessor(self.struct, FJBIAM, JBIAMJ, QBMJVH),
            BIAMJM: GeckoTempStructAccessor(self.struct, BIAMJM, IAMJMA, QBMJVH),
            AMJMAO: GeckoEnumStructAccessor(
                self.struct, AMJMAO, MJMAOA, TYEKCW, AOAWBS, None, TYEKCW, QBMJVH
            ),
            OAWBSI: GeckoEnumStructAccessor(
                self.struct, OAWBSI, AWBSIR, None, JYKLGQ, None, None, QBMJVH
            ),
            WBSIRY: GeckoEnumStructAccessor(
                self.struct, WBSIRY, BSIRYX, YXBQFY, RYXBQF, None, TYEKCW, QBMJVH
            ),
            XBQFYL: GeckoBoolStructAccessor(
                self.struct, XBQFYL, BQFYLJ, FEFJTA, QBMJVH
            ),
            QFYLJU: GeckoBoolStructAccessor(
                self.struct, QFYLJU, BQFYLJ, FYLJUI, QBMJVH
            ),
            YLJUIK: GeckoByteStructAccessor(self.struct, YLJUIK, LJUIKF, QBMJVH),
            JUIKFW: GeckoEnumStructAccessor(
                self.struct, JUIKFW, BSIRYX, FWRKIN, KFWRKI, None, TYEKCW, QBMJVH
            ),
            WRKINE: GeckoEnumStructAccessor(
                self.struct, WRKINE, BSIRYX, TYEKCW, INEJNI, None, TYEKCW, QBMJVH
            ),
            NEJNIB: GeckoByteStructAccessor(self.struct, NEJNIB, EJNIBX, None),
            JNIBXY: GeckoByteStructAccessor(self.struct, JNIBXY, NIBXYB, None),
            IBXYBQ: GeckoByteStructAccessor(self.struct, IBXYBQ, BXYBQS, None),
            XYBQSN: GeckoByteStructAccessor(self.struct, XYBQSN, YBQSNQ, QBMJVH),
            BQSNQL: GeckoByteStructAccessor(self.struct, BQSNQL, QSNQLN, QBMJVH),
            SNQLNM: GeckoByteStructAccessor(self.struct, SNQLNM, NQLNMH, QBMJVH),
            QLNMHX: GeckoByteStructAccessor(self.struct, QLNMHX, LNMHXE, QBMJVH),
            NMHXEK: GeckoByteStructAccessor(self.struct, NMHXEK, MHXEKV, QBMJVH),
            HXEKVK: GeckoBoolStructAccessor(
                self.struct, HXEKVK, XEKVKZ, POUYNQ, QBMJVH
            ),
            EKVKZI: GeckoEnumStructAccessor(
                self.struct, EKVKZI, XEKVKZ, FWRKIN, ZILXWA, None, FEFJTA, QBMJVH
            ),
            ILXWAJ: GeckoEnumStructAccessor(
                self.struct, ILXWAJ, XEKVKZ, QJYMOU, WAJVDQ, None, TYEKCW, QBMJVH
            ),
            AJVDQL: GeckoEnumStructAccessor(
                self.struct, AJVDQL, MJMAOA, YXBQFY, QLAIID, None, FEFJTA, QBMJVH
            ),
            LAIIDN: GeckoBoolStructAccessor(
                self.struct, LAIIDN, AIIDNI, QJYMOU, QBMJVH
            ),
            IIDNIB: GeckoBoolStructAccessor(
                self.struct, IIDNIB, AIIDNI, TYEKCW, QBMJVH
            ),
            IDNIBX: GeckoByteStructAccessor(self.struct, IDNIBX, DNIBXT, QBMJVH),
            NIBXTI: GeckoByteStructAccessor(self.struct, NIBXTI, IBXTIA, QBMJVH),
            BXTIAC: GeckoByteStructAccessor(self.struct, BXTIAC, XTIACQ, QBMJVH),
            TIACQF: GeckoByteStructAccessor(self.struct, TIACQF, IACQFF, QBMJVH),
            ACQFFT: GeckoByteStructAccessor(self.struct, ACQFFT, CQFFTT, QBMJVH),
            QFFTTI: GeckoByteStructAccessor(self.struct, QFFTTI, FFTTID, QBMJVH),
            FTTIDU: GeckoByteStructAccessor(self.struct, FTTIDU, TTIDUB, QBMJVH),
            TIDUBS: GeckoTimeStructAccessor(self.struct, TIDUBS, IDUBSS, QBMJVH),
            DUBSSU: GeckoTimeStructAccessor(self.struct, DUBSSU, UBSSUH, QBMJVH),
            BSSUHB: GeckoTimeStructAccessor(self.struct, BSSUHB, SSUHBV, QBMJVH),
            SUHBVW: GeckoTimeStructAccessor(self.struct, SUHBVW, UHBVWV, QBMJVH),
            HBVWVU: GeckoByteStructAccessor(self.struct, HBVWVU, BVWVUB, QBMJVH),
            VWVUBY: GeckoTimeStructAccessor(self.struct, VWVUBY, WVUBYG, QBMJVH),
            VUBYGD: GeckoTimeStructAccessor(self.struct, VUBYGD, UBYGDS, QBMJVH),
            BYGDSB: GeckoEnumStructAccessor(
                self.struct, BYGDSB, XEKVKZ, FEFJTA, GDSBDJ, None, TYEKCW, QBMJVH
            ),
            DSBDJQ: GeckoBoolStructAccessor(
                self.struct, DSBDJQ, XEKVKZ, FYLJUI, QBMJVH
            ),
            SBDJQR: GeckoBoolStructAccessor(
                self.struct, SBDJQR, XEKVKZ, YXBQFY, QBMJVH
            ),
            BDJQRJ: GeckoEnumStructAccessor(
                self.struct, BDJQRJ, XUJUTY, QJYMOU, QRJJJV, None, TYEKCW, QBMJVH
            ),
            RJJJVY: GeckoBoolStructAccessor(
                self.struct, RJJJVY, XUJUTY, FWRKIN, QBMJVH
            ),
            XNKMLO: GeckoEnumStructAccessor(
                self.struct, XNKMLO, BSIRYX, QJYMOU, MLOIJU, None, TYEKCW, QBMJVH
            ),
            LOIJUG: GeckoBoolStructAccessor(
                self.struct, LOIJUG, BQFYLJ, QJYMOU, QBMJVH
            ),
            OIJUGS: GeckoBoolStructAccessor(
                self.struct, OIJUGS, XFEFJT, FWRKIN, QBMJVH
            ),
            IJUGSE: GeckoBoolStructAccessor(
                self.struct, IJUGSE, JUGSEL, QJYMOU, QBMJVH
            ),
            UGSELH: GeckoEnumStructAccessor(
                self.struct, UGSELH, JUGSEL, JJVYFC, ELHBQN, None, TYEKCW, QBMJVH
            ),
            LHBQNR: GeckoEnumStructAccessor(
                self.struct, LHBQNR, JUGSEL, FEFJTA, WDAFIK, None, DAFIKJ, QBMJVH
            ),
            OIHBXI: GeckoEnumStructAccessor(
                self.struct, OIHBXI, JUGSEL, TYEKCW, BXIBHZ, None, TYEKCW, QBMJVH
            ),
            XIBHZV: GeckoEnumStructAccessor(
                self.struct, XIBHZV, JUGSEL, YXBQFY, BXIBHZ, None, TYEKCW, QBMJVH
            ),
            IBHZVO: GeckoBoolStructAccessor(
                self.struct, IBHZVO, BHZVOA, QJYMOU, QBMJVH
            ),
            HZVOAC: GeckoEnumStructAccessor(
                self.struct, HZVOAC, ZVOACM, TYEKCW, ACMCVD, None, FEFJTA, QBMJVH
            ),
            CMCVDS: GeckoByteStructAccessor(self.struct, CMCVDS, ZVOACM, QBMJVH),
            MCVDSS: GeckoBoolStructAccessor(
                self.struct, MCVDSS, BSIRYX, FEFJTA, QBMJVH
            ),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InXE v10'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = 32
AHEOCT = "".join(chr(c) for c in [79, 117, 116, 53, 67, 117, 114])
AKQXPI = 11
AONPYY = 21
ASSAKQ = "".join(chr(c) for c in [79, 117, 116, 50])
BFEGZU = "".join(chr(c) for c in [79, 51, 85, 115, 97, 103, 101])
BLKXSJ = 55
BMJVHF = "".join(chr(c) for c in [79, 117, 116, 49])
BSKSOK = 34
BWJYKL = "".join(
    chr(c) for c in [78, 111, 66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67]
)
CBFEGZ = 46
CCPQIP = "".join(
    chr(c) for c in [73, 110, 112, 117, 116, 67, 117, 114, 114, 101, 110, 116]
)
CPQIPO = 33
CQBMJV = 0
CTHBSK = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 67, 117, 114])
CVYYPI = "".join(chr(c) for c in [80, 52, 72])
CWAONP = 19
CXQIEF = 14
ECVYYP = "".join(chr(c) for c in [80, 51, 76])
EFJTAC = 54
EFXQGL = 24
EGZUQE = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114])
EKCWAO = 1
EOCTHB = "".join(chr(c) for c in [79, 117, 116, 72, 116, 82, 67, 117, 114])
EXLSXU = "".join(chr(c) for c in [80, 49])
FEFJTA = "".join(chr(c) for c in [69, 99, 111, 110, 84, 121, 112, 101])
FEGZUQ = 16
FJTACC = "".join(chr(c) for c in [78, 105, 103, 104, 116])
FTHECV = "".join(chr(c) for c in [80, 50, 72])
FXQGLR = "".join(chr(c) for c in [79, 117, 116, 50, 67, 117, 114])
GLRAHE = 26
GQPLSP = "".join(chr(c) for c in [50, 52, 104])
GYOUSP = "".join(chr(c) for c in [83, 108, 97, 118, 101])
GZUQEX = "".join(chr(c) for c in [65, 108, 119, 97, 121, 115])
HBSKSO = "".join(chr(c) for c in [80, 117, 109, 112, 84, 105, 109, 101, 79, 117, 116])
HECVYY = "".join(chr(c) for c in [80, 51, 72])
HEOCTH = 28
HFTHEC = "".join(chr(c) for c in [80, 49, 76])
HIUSOO = 0
HUOJRJ = "".join(chr(c) for c in [80, 117, 114, 103, 101, 79, 110, 108, 121])
ICXQIE = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114])
IEFXQG = "".join(chr(c) for c in [79, 117, 116, 49, 67, 117, 114])
IGYOUS = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114])
IPIVLA = "".join(chr(c) for c in [79, 51])
IPOUYN = "".join(chr(c) for c in [68, 117, 97, 108, 80, 97, 99, 107])
IUSOOQ = "".join(
    chr(c) for c in [79, 84, 65, 99, 116, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
IUXFEF = 50
IVLASS = "".join(chr(c) for c in [])
JHIUSO = 15
JIGYOU = "".join(
    chr(c) for c in [85, 110, 99, 111, 110, 102, 105, 103, 117, 114, 101, 100]
)
JMCBFE = 44
JVHFTH = "".join(chr(c) for c in [78, 65])
JWMNZM = "".join(
    chr(c) for c in [79, 102, 102, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
JYMOUN = 3
KCWAON = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 117, 109, 112])
KLGQPL = 22
KPHUOJ = "".join(
    chr(c) for c in [70, 105, 108, 116, 73, 110, 116, 101, 114, 102, 97, 99, 101]
)
KQXPIC = "".join(chr(c) for c in [79, 117, 116, 52])
KSOKPH = 35
LGQPLS = "".join(chr(c) for c in [65, 109, 80, 109])
LIUXFE = "".join(
    chr(c) for c in [77, 105, 110, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
LKXSJW = "".join(chr(c) for c in [65, 118, 97, 105, 108, 97, 98, 108, 101])
LRAHEO = "".join(chr(c) for c in [79, 117, 116, 52, 67, 117, 114])
LSPFTS = 48
LSXUJU = "".join(chr(c) for c in [79, 51, 84, 121, 112, 101])
MCBFEG = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
MJIGYO = 57
MJVHFT = 9
MNZMJI = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        67,
        111,
        110,
        116,
        114,
        111,
        108,
        97,
        98,
        108,
        101,
        77,
        97,
        110,
        117,
        97,
        108,
        108,
        121,
    ]
)
MOUNBL = 7
NBLKXS = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        80,
        114,
        111,
        103,
        65,
        118,
        97,
        105,
        108,
        97,
        98,
        108,
        101,
    ]
)
NPYYLI = "".join(chr(c) for c in [67])
NQJYMO = 5
NRSJMC = "".join(
    chr(c) for c in [67, 80, 79, 84, 77, 97, 120, 79, 102, 102, 84, 105, 109, 101]
)
NZMJIG = 2
OCTHBS = 29
OJRJHI = "".join(chr(c) for c in [70, 105, 108, 116, 80, 49])
OKPHUO = 36
ONPYYL = "".join(chr(c) for c in [70])
OOQNRS = 39
OQNRSJ = "".join(
    chr(c) for c in [67, 80, 79, 84, 77, 97, 120, 79, 110, 84, 105, 109, 101]
)
OUNBLK = "".join(chr(c) for c in [69, 99, 111, 110, 68, 117, 114])
OUSPBW = "".join(chr(c) for c in [83, 108, 97, 118, 101, 67, 111, 110, 102, 105, 103])
OUYNQJ = "".join(chr(c) for c in [70, 105, 108, 116, 70, 114, 101, 113])
PBWJYK = 60
PFTSIF = "".join(chr(c) for c in [76, 73])
PHUOJR = 20
PICXQI = 13
PIPIVL = "".join(chr(c) for c in [67, 80])
PIVLAS = "".join(chr(c) for c in [76, 49, 50, 48])
PLSPFT = "".join(
    chr(c)
    for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 84, 114, 105, 103, 65, 68, 67]
)
PQIPOU = "".join(chr(c) for c in [73, 110, 112, 117, 116, 77, 101, 110, 117])
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QEXLSX = 17
QGLRAH = "".join(chr(c) for c in [79, 117, 116, 51, 67, 117, 114])
QIPOUY = 58
QJYMOU = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114])
QNRSJM = 40
QXPICX = 12
RAHEOC = 27
RJHIUS = "".join(chr(c) for c in [67, 80, 65, 108, 119, 97, 121, 115, 79, 78])
RSJMCB = 42
SAKQXP = "".join(chr(c) for c in [79, 117, 116, 51])
SIFJBI = 10
SJMCBF = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        79,
        84,
        68,
        117,
        114,
        97,
        116,
        105,
        111,
        110,
        50,
        52,
        72,
    ]
)
SJWMNZ = 56
SKSOKP = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 84, 105, 109, 101, 79, 117, 116]
)
SOKPHU = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 79, 117, 116])
SOOQNR = "".join(chr(c) for c in [79, 84, 84, 114, 105, 103, 103, 101, 114, 71])
SPBWJY = "".join(
    chr(c) for c in [77, 117, 108, 116, 105, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
SSAKQX = 10
SXUJUT = 18
TACCPQ = "".join(chr(c) for c in [78, 98, 80, 104, 97, 115, 101, 115])
THBSKS = 30
THECVY = "".join(chr(c) for c in [80, 50, 76])
TYEKCW = 47
UJUTYE = "".join(chr(c) for c in [84, 111, 103, 103, 108, 101])
UNBLKX = 8
UOJRJH = "".join(chr(c) for c in [70, 105, 108, 116, 67, 80])
UQEXLS = "".join(chr(c) for c in [79, 51, 80, 117, 109, 112])
USOOQN = 37
USPBWJ = 59
UTYEKC = "".join(
    chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
UXFEFJ = "".join(
    chr(c) for c in [77, 97, 120, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
UYNQJY = 4
VHFTHE = "".join(chr(c) for c in [80, 49, 72])
VLASSA = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
VYYPIP = "".join(chr(c) for c in [80, 52, 76])
WAONPY = "".join(chr(c) for c in [84, 101, 109, 112, 85, 110, 105, 116, 115])
WJYKLG = "".join(chr(c) for c in [66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67])
WMNZMJ = 1
XFEFJT = 52
XPICXQ = "".join(chr(c) for c in [79, 117, 116, 53])
XQGLRA = 25
XQIEFX = "".join(chr(c) for c in [72, 84, 82])
XSJWMN = "".join(
    chr(c)
    for c in [83, 111, 97, 107, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
XUJUTY = "".join(chr(c) for c in [83, 116, 97, 110, 100, 97, 114, 100])
YEKCWA = "".join(chr(c) for c in [83, 101, 116, 112, 111, 105, 110, 116, 71])
YKLGQP = "".join(chr(c) for c in [84, 105, 109, 101, 70, 111, 114, 109, 97, 116])
YLIUXF = 23
YMOUNB = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 97, 114, 116])
YNQJYM = "".join(chr(c) for c in [70, 105, 108, 116, 83, 116, 97, 114, 116])
YPIPIV = "".join(chr(c) for c in [66, 76, 79])
YYLIUX = "".join(
    chr(c) for c in [67, 111, 111, 108, 100, 111, 119, 110, 84, 105, 109, 101]
)
YYPIPI = "".join(chr(c) for c in [80, 53])
ZCQBMJ = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 78, 117, 109, 98, 101, 114]
)
ZMJIGY = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114, 83, 108, 97, 118, 101])
FTSIFJ = [PFTSIF]
JRJHIU = [HUOJRJ, UOJRJH, OJRJHI]
JTACCP = [XUJUTY, FJTACC]
JUTYEK = [XUJUTY, UJUTYE]
JYKLGQ = [BWJYKL, WJYKLG]
KXSJWM = [JVHFTH, LKXSJW]
LASSAK = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    VLASSA,
]
POUYNQ = [XUJUTY, IPOUYN]
PYYLIU = [ONPYYL, NPYYLI]
QIEFXQ = [
    JVHFTH,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    IVLASS,
    XQIEFX,
]
QPLSPF = [JVHFTH, LGQPLS, GQPLSP]
SPFTSI = [BMJVHF, ASSAKQ, SAKQXP, KQXPIC, XPICXQ, ICXQIE]
TSIFJB = []
XLSXUJ = [PIPIVL, EXLSXU]
YOUSPB = [JIGYOU, IGYOUS, GYOUSP]
ZUQEXL = [EGZUQE, GZUQEX]


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return SIFJBI

    @property
    def output_keys(self):
        return SPFTSI

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoEnumStructAccessor(
                self.struct, BMJVHF, MJVHFT, None, LASSAK, None, None, QBMJVH
            ),
            ASSAKQ: GeckoEnumStructAccessor(
                self.struct, ASSAKQ, SSAKQX, None, LASSAK, None, None, QBMJVH
            ),
            SAKQXP: GeckoEnumStructAccessor(
                self.struct, SAKQXP, AKQXPI, None, LASSAK, None, None, QBMJVH
            ),
            KQXPIC: GeckoEnumStructAccessor(
                self.struct, KQXPIC, QXPICX, None, LASSAK, None, None, QBMJVH
            ),
            XPICXQ: GeckoEnumStructAccessor(
                self.struct, XPICXQ, PICXQI, None, LASSAK, None, None, QBMJVH
            ),
            ICXQIE: GeckoEnumStructAccessor(
                self.struct, ICXQIE, CXQIEF, None, QIEFXQ, None, None, QBMJVH
            ),
            IEFXQG: GeckoByteStructAccessor(self.struct, IEFXQG, EFXQGL, QBMJVH),
            FXQGLR: GeckoByteStructAccessor(self.struct, FXQGLR, XQGLRA, QBMJVH),
            QGLRAH: GeckoByteStructAccessor(self.struct, QGLRAH, GLRAHE, QBMJVH),
            LRAHEO: GeckoByteStructAccessor(self.struct, LRAHEO, RAHEOC, QBMJVH),
            AHEOCT: GeckoByteStructAccessor(self.struct, AHEOCT, HEOCTH, QBMJVH),
            EOCTHB: GeckoByteStructAccessor(self.struct, EOCTHB, OCTHBS, QBMJVH),
            CTHBSK: GeckoByteStructAccessor(self.struct, CTHBSK, THBSKS, QBMJVH),
            HBSKSO: GeckoByteStructAccessor(self.struct, HBSKSO, BSKSOK, QBMJVH),
            SKSOKP: GeckoByteStructAccessor(self.struct, SKSOKP, KSOKPH, QBMJVH),
            SOKPHU: GeckoByteStructAccessor(self.struct, SOKPHU, OKPHUO, QBMJVH),
            KPHUOJ: GeckoEnumStructAccessor(
                self.struct, KPHUOJ, PHUOJR, None, JRJHIU, None, None, QBMJVH
            ),
            RJHIUS: GeckoBoolStructAccessor(
                self.struct, RJHIUS, JHIUSO, HIUSOO, QBMJVH
            ),
            IUSOOQ: GeckoTempStructAccessor(self.struct, IUSOOQ, USOOQN, QBMJVH),
            SOOQNR: GeckoByteStructAccessor(self.struct, SOOQNR, OOQNRS, QBMJVH),
            OQNRSJ: GeckoWordStructAccessor(self.struct, OQNRSJ, QNRSJM, QBMJVH),
            NRSJMC: GeckoWordStructAccessor(self.struct, NRSJMC, RSJMCB, QBMJVH),
            SJMCBF: GeckoWordStructAccessor(self.struct, SJMCBF, JMCBFE, QBMJVH),
            MCBFEG: GeckoByteStructAccessor(self.struct, MCBFEG, CBFEGZ, QBMJVH),
            BFEGZU: GeckoEnumStructAccessor(
                self.struct, BFEGZU, FEGZUQ, None, ZUQEXL, None, None, QBMJVH
            ),
            UQEXLS: GeckoEnumStructAccessor(
                self.struct, UQEXLS, QEXLSX, None, XLSXUJ, None, None, QBMJVH
            ),
            LSXUJU: GeckoEnumStructAccessor(
                self.struct, LSXUJU, SXUJUT, None, JUTYEK, None, None, QBMJVH
            ),
            UTYEKC: GeckoByteStructAccessor(self.struct, UTYEKC, TYEKCW, QBMJVH),
            YEKCWA: GeckoTempStructAccessor(self.struct, YEKCWA, EKCWAO, QBMJVH),
            KCWAON: GeckoEnumStructAccessor(
                self.struct, KCWAON, CWAONP, None, XLSXUJ, None, None, QBMJVH
            ),
            WAONPY: GeckoEnumStructAccessor(
                self.struct, WAONPY, AONPYY, None, PYYLIU, None, None, QBMJVH
            ),
            YYLIUX: GeckoByteStructAccessor(self.struct, YYLIUX, YLIUXF, QBMJVH),
            LIUXFE: GeckoTempStructAccessor(self.struct, LIUXFE, IUXFEF, QBMJVH),
            UXFEFJ: GeckoTempStructAccessor(self.struct, UXFEFJ, XFEFJT, QBMJVH),
            FEFJTA: GeckoEnumStructAccessor(
                self.struct, FEFJTA, EFJTAC, None, JTACCP, None, None, QBMJVH
            ),
            TACCPQ: GeckoByteStructAccessor(self.struct, TACCPQ, ACCPQI, QBMJVH),
            CCPQIP: GeckoByteStructAccessor(self.struct, CCPQIP, CPQIPO, QBMJVH),
            PQIPOU: GeckoEnumStructAccessor(
                self.struct, PQIPOU, QIPOUY, None, POUYNQ, None, None, QBMJVH
            ),
            OUYNQJ: GeckoByteStructAccessor(self.struct, OUYNQJ, UYNQJY, QBMJVH),
            YNQJYM: GeckoTimeStructAccessor(self.struct, YNQJYM, NQJYMO, QBMJVH),
            QJYMOU: GeckoTimeStructAccessor(self.struct, QJYMOU, JYMOUN, QBMJVH),
            YMOUNB: GeckoTimeStructAccessor(self.struct, YMOUNB, MOUNBL, QBMJVH),
            OUNBLK: GeckoTimeStructAccessor(self.struct, OUNBLK, UNBLKX, QBMJVH),
            NBLKXS: GeckoEnumStructAccessor(
                self.struct, NBLKXS, BLKXSJ, None, KXSJWM, None, None, QBMJVH
            ),
            XSJWMN: GeckoBoolStructAccessor(
                self.struct, XSJWMN, SJWMNZ, HIUSOO, QBMJVH
            ),
            JWMNZM: GeckoBoolStructAccessor(
                self.struct, JWMNZM, SJWMNZ, WMNZMJ, QBMJVH
            ),
            MNZMJI: GeckoBoolStructAccessor(
                self.struct, MNZMJI, SJWMNZ, NZMJIG, QBMJVH
            ),
            ZMJIGY: GeckoEnumStructAccessor(
                self.struct, ZMJIGY, MJIGYO, None, YOUSPB, None, None, QBMJVH
            ),
            OUSPBW: GeckoByteStructAccessor(self.struct, OUSPBW, USPBWJ, QBMJVH),
            SPBWJY: GeckoEnumStructAccessor(
                self.struct, SPBWJY, PBWJYK, None, JYKLGQ, None, None, QBMJVH
            ),
            YKLGQP: GeckoEnumStructAccessor(
                self.struct, YKLGQP, KLGQPL, None, QPLSPF, None, None, QBMJVH
            ),
            PLSPFT: GeckoWordStructAccessor(self.struct, PLSPFT, LSPFTS, QBMJVH),
        }



#!/usr/bin/python3
"""
    GeckoConfigStruct - A class to manage the ConfigStruct for 'InYT v55'
"""

from . import (
    GeckoByteStructAccessor,
    GeckoWordStructAccessor,
    GeckoTimeStructAccessor,
    GeckoBoolStructAccessor,
    GeckoEnumStructAccessor,
    GeckoTempStructAccessor,
)

# Constants for this class
ACCPQI = "".join(chr(c) for c in [67, 112, 85, 115, 97, 103, 101])
ACQFFT = 96
AHEOCT = "".join(chr(c) for c in [79, 117, 116, 53, 67, 117, 114])
AIIDNI = 91
AJVDQL = 88
AKQXPI = "".join(chr(c) for c in [79, 117, 116, 51])
AMJMAO = "".join(
    chr(c) for c in [80, 114, 101, 115, 115, 117, 114, 101, 83, 119, 105, 116, 99, 104]
)
AOAWBS = "".join(chr(c) for c in [73, 110, 116, 111, 84, 117, 98])
AONPYY = 56
AWBSIR = 3
BDJQRJ = 108
BFEGZU = "".join(chr(c) for c in [79, 117, 116, 49, 49, 67, 117, 114])
BIAMJM = "".join(chr(c) for c in [105, 110, 70, 108, 111])
BJEUTO = "".join(chr(c) for c in [85, 68, 80, 114, 111, 103, 69, 99, 111, 110])
BLKXSJ = 59
BMJVHF = "".join(chr(c) for c in [79, 117, 116, 49])
BQFYLJ = 70
BQNRXC = 7
BQSNQL = "".join(chr(c) for c in [70, 49])
BSIRYX = 35
BSKSOK = 18
BSSUHB = 101
BVWVUB = "".join(chr(c) for c in [70, 49, 76, 105, 110, 101])
BWJYKL = "".join(chr(c) for c in [79, 51, 84, 121, 112, 101])
BXTIAC = 94
BYGDSB = "".join(chr(c) for c in [70, 51, 76, 105, 110, 101])
CBFEGZ = 45
CCPQIP = 27
CHWDAF = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 51]
)
CPQIPO = "".join(chr(c) for c in [83, 84, 65, 78, 68, 65, 82, 68])
CQBMJV = 0
CQFFTT = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114, 70, 117, 115, 101])
CRTFMN = "".join(chr(c) for c in [69, 99, 111, 110, 83, 116, 97, 114, 116])
CTHBSK = "".join(chr(c) for c in [79, 117, 116, 54])
CVYYPI = "".join(chr(c) for c in [80, 52, 72])
CWAONP = 55
CXQIEF = "".join(chr(c) for c in [79, 117, 116, 72, 116, 114])
DAFIKJ = 113
DJQRJJ = "".join(chr(c) for c in [70, 50, 51, 76, 105, 110, 101])
DNIBXT = "".join(chr(c) for c in [79, 117, 116, 49, 49, 70, 117, 115, 101])
DNQGVU = "".join(chr(c) for c in [84, 105, 109, 101, 70, 111, 114, 109, 97, 116])
DQLAII = "".join(chr(c) for c in [79, 117, 116, 56, 70, 117, 115, 101])
DSBDJQ = 107
DUBSSU = 100
ECVYYP = "".join(chr(c) for c in [80, 51, 76])
EFJTAC = "".join(chr(c) for c in [70, 105, 108, 116, 67, 80])
EFXQGL = 36
EGZUQE = "".join(chr(c) for c in [79, 117, 116, 49, 50, 67, 117, 114])
EJNIBX = 53
EKCWAO = 118
EKVKZI = "".join(chr(c) for c in [79, 117, 116, 50, 70, 117, 115, 101])
ELHBQN = "".join(
    chr(c)
    for c in [72, 101, 97, 116, 101, 114, 83, 111, 102, 116, 83, 116, 97, 114, 116]
)
EOCTHB = "".join(chr(c) for c in [79, 117, 116, 72, 116, 82, 67, 117, 114])
EUTOPH = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        67,
        111,
        110,
        116,
        114,
        111,
        108,
        97,
        98,
        108,
        101,
        77,
        97,
        110,
        117,
        97,
        108,
        108,
        121,
    ]
)
EXLSXU = 49
FCRTFM = 116
FEFJTA = "".join(chr(c) for c in [80, 117, 114, 103, 101, 79, 110, 108, 121])
FEGZUQ = 46
FFTTID = "".join(chr(c) for c in [70, 49, 67, 117, 114, 114, 101, 110, 116])
FJBIAM = "".join(
    chr(c) for c in [70, 108, 111, 119, 68, 101, 116, 101, 99, 116, 111, 114]
)
FJTACC = "".join(chr(c) for c in [70, 105, 108, 116, 80, 49])
FMNHTB = 10
FTHECV = "".join(chr(c) for c in [80, 50, 72])
FTSIFJ = "".join(chr(c) for c in [67])
FTTIDU = 98
FWRKIN = "".join(chr(c) for c in [85, 76])
FXQGLR = "".join(chr(c) for c in [79, 117, 116, 50, 67, 117, 114])
GDSBDJ = "".join(chr(c) for c in [70, 50, 49, 76, 105, 110, 101])
GLRAHE = 38
GQPLSP = 63
GTYIYW = 76
GVUNXN = "".join(chr(c) for c in [50, 52, 104])
GYOUSP = "".join(chr(c) for c in [70, 105, 108, 116, 101, 114])
GZUQEX = 47
HBQNRX = "".join(
    chr(c) for c in [72, 101, 97, 116, 101, 114, 83, 111, 102, 116, 83, 116, 111, 112]
)
HBSKSO = "".join(chr(c) for c in [79, 117, 116, 55])
HBVWVU = 103
HECVYY = "".join(chr(c) for c in [80, 51, 72])
HEOCTH = 40
HFTHEC = "".join(chr(c) for c in [80, 49, 76])
HTBJEU = "".join(
    chr(c) for c in [79, 85, 84, 83, 73, 68, 69, 95, 70, 73, 76, 84, 69, 82]
)
HUGTYI = "".join(
    chr(c)
    for c in [
        67,
        108,
        101,
        97,
        110,
        117,
        112,
        79,
        110,
        67,
        117,
        115,
        116,
        111,
        109,
        75,
        101,
        121,
    ]
)
HUOJRJ = "".join(chr(c) for c in [79, 117, 116, 49, 49])
HWDAFI = 112
HXEKVK = "".join(chr(c) for c in [76, 105, 110, 101, 51])
IACQFF = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 50, 70, 117, 115, 101])
IAMJMA = "".join(
    chr(c) for c in [78, 111, 116, 73, 110, 115, 116, 97, 108, 108, 101, 100]
)
IBXTIA = "".join(chr(c) for c in [79, 117, 116, 49, 50, 70, 117, 115, 101])
IBXYBQ = "".join(chr(c) for c in [68, 117, 97, 108, 80, 97, 99, 107])
ICXQIE = 16
IDNIBX = 92
IDUBSS = "".join(chr(c) for c in [70, 51, 67, 117, 114, 114, 101, 110, 116])
IEFXQG = "".join(chr(c) for c in [79, 117, 116, 49, 67, 117, 114])
IFJBIA = 31
IGYOUS = 28
IIDNIB = "".join(chr(c) for c in [79, 117, 116, 49, 48, 70, 117, 115, 101])
IJUGSE = "".join(
    chr(c)
    for c in [66, 108, 111, 119, 101, 114, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
IKFWRK = "".join(chr(c) for c in [85, 76, 95, 67, 69])
ILXWAJ = 86
INEJNI = 52
IPIVLA = "".join(chr(c) for c in [79, 51])
IPOUYN = "".join(chr(c) for c in [79, 116, 79, 112, 116, 105, 111, 110])
IRYXBQ = 66
IUSOOQ = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 50])
IUXFEF = 2
IVDNQG = "".join(chr(c) for c in [83, 108, 97, 118, 101, 67, 111, 110, 102, 105, 103])
IVLASS = "".join(chr(c) for c in [72, 84, 82])
JBIAMJ = 115
JEUTOP = 82
JHIUSO = 24
JIGYOU = "".join(chr(c) for c in [79, 51, 85, 115, 97, 103, 101])
JJJVYF = "".join(chr(c) for c in [70, 105, 108, 116, 83, 116, 97, 114, 116])
JJVYFC = 4
JMAOAW = "".join(
    chr(c) for c in [80, 114, 111, 98, 101, 76, 111, 99, 97, 116, 105, 111, 110]
)
JMCBFE = 44
JNIBXY = "".join(chr(c) for c in [73, 110, 112, 117, 116, 77, 101, 110, 117])
JQRJJJ = 109
JRJHIU = 23
JTACCP = "".join(
    chr(c) for c in [70, 105, 108, 116, 80, 49, 68, 117, 114, 79, 110, 108, 121]
)
JUGSEL = "".join(chr(c) for c in [70, 114, 101, 101, 80, 117, 109, 112, 75, 101, 121])
JUIKFW = "".join(
    chr(c)
    for c in [
        77,
        97,
        120,
        78,
        117,
        109,
        98,
        101,
        114,
        79,
        102,
        80,
        104,
        97,
        115,
        101,
        115,
    ]
)
JUTYEK = 80
JVDQLA = "".join(chr(c) for c in [79, 117, 116, 55, 70, 117, 115, 101])
JVHFTH = "".join(chr(c) for c in [78, 65])
JVYFCR = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114])
JWMNZM = "".join(
    chr(c)
    for c in [70, 105, 108, 116, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
JYKLGQ = "".join(chr(c) for c in [83, 116, 97, 110, 100, 97, 114, 100])
JYMOUN = "".join(chr(c) for c in [76, 111])
KCWAON = "".join(
    chr(c) for c in [76, 105, 103, 104, 116, 84, 105, 109, 101, 79, 117, 116]
)
KFWRKI = 51
KINEJN = "".join(chr(c) for c in [78, 98, 80, 104, 97, 115, 101, 115])
KJPUNR = 55
KMLOIJ = "".join(
    chr(c) for c in [72, 105, 103, 104, 83, 112, 101, 101, 100, 79, 110, 108, 121]
)
KPHUOJ = "".join(chr(c) for c in [79, 117, 116, 49, 48])
KQXPIC = 14
KSOKPH = 19
KVKZIL = 84
KWIVDN = "".join(chr(c) for c in [83, 108, 97, 118, 101])
KXSJWM = 60
KZILXW = 85
LAIIDN = "".join(chr(c) for c in [79, 117, 116, 57, 70, 117, 115, 101])
LASSAK = "".join(chr(c) for c in [70, 117, 108, 108, 79, 110])
LGQPLS = "".join(
    chr(c) for c in [79, 51, 83, 117, 115, 112, 101, 110, 100, 84, 105, 109, 101]
)
LHBQNR = 6
LIUXFE = 0
LJUIKF = 77
LKXSJW = "".join(
    chr(c)
    for c in [
        67,
        112,
        79,
        102,
        102,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
LNMHXE = "".join(chr(c) for c in [70, 50, 51])
LOIJUG = "".join(
    chr(c) for c in [83, 101, 108, 102, 67, 108, 101, 97, 110, 77, 115, 103]
)
LRAHEO = "".join(chr(c) for c in [79, 117, 116, 52, 67, 117, 114])
LSPFTS = "".join(chr(c) for c in [84, 101, 109, 112, 85, 110, 105, 116, 115])
LSXUJU = 79
LXWAJV = "".join(chr(c) for c in [79, 117, 116, 53, 70, 117, 115, 101])
MAOAWB = "".join(chr(c) for c in [73, 110, 116, 111, 80, 105, 112, 105, 110, 103])
MCBFEG = "".join(chr(c) for c in [79, 117, 116, 49, 48, 67, 117, 114])
MHXEKV = "".join(chr(c) for c in [76, 105, 110, 101, 50])
MJVHFT = 12
MNHTBJ = "".join(
    chr(c)
    for c in [
        69,
        99,
        111,
        110,
        80,
        114,
        111,
        103,
        65,
        118,
        97,
        105,
        108,
        97,
        98,
        108,
        101,
    ]
)
MNZMJI = "".join(chr(c) for c in [68, 114, 97, 105, 110, 77, 111, 100, 101])
NBLKXS = "".join(
    chr(c)
    for c in [67, 112, 79, 110, 84, 105, 109, 101, 68, 117, 114, 105, 110, 103, 79, 84]
)
NEJNIB = "".join(
    chr(c) for c in [73, 110, 112, 117, 116, 67, 117, 114, 114, 101, 110, 116]
)
NHTBJE = 71
NIBXTI = 93
NIBXYB = 74
NKMLOI = "".join(chr(c) for c in [66, 111, 116, 104, 83, 112, 101, 101, 100, 115])
NMHXEK = "".join(chr(c) for c in [76, 105, 110, 101, 49])
NPYYLI = 81
NQGVUN = 34
NQLNMH = "".join(chr(c) for c in [70, 50, 49])
NRSJMC = "".join(chr(c) for c in [79, 117, 116, 56, 67, 117, 114])
NRXCHW = 110
NXNKML = 64
NZMJIG = 78
OCTHBS = 50
OIJUGS = 4
OJRJHI = "".join(chr(c) for c in [79, 117, 116, 49, 50])
OKPHUO = 20
ONPYYL = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 114])
OOQNRS = 41
OPHUGT = "".join(
    chr(c) for c in [79, 102, 102, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
OQNRSJ = "".join(chr(c) for c in [79, 117, 116, 55, 67, 117, 114])
OUNBLK = "".join(chr(c) for c in [79, 84, 84, 114, 105, 103, 103, 101, 114, 71])
OUYNQJ = "".join(chr(c) for c in [68, 105, 115, 97, 98, 108, 101, 100])
PFTSIF = "".join(chr(c) for c in [70])
PHUGTY = 1
PHUOJR = 21
PICXQI = "".join(chr(c) for c in [79, 117, 116, 53])
PIPIVL = "".join(chr(c) for c in [67, 80])
PIVLAS = "".join(chr(c) for c in [76, 49, 50, 48])
PLSPFT = 1
POUYNQ = 57
PQIPOU = "".join(chr(c) for c in [65, 76, 87, 65, 89, 83, 95, 79, 78])
PYYLIU = "".join(chr(c) for c in [83, 104, 97, 114, 101, 100])
QBMJVH = "".join(chr(c) for c in [65, 76, 76])
QEXLSX = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 50, 67, 117, 114])
QFFTTI = 97
QFYLJU = "".join(chr(c) for c in [78, 105, 103, 104, 116])
QGLRAH = "".join(chr(c) for c in [79, 117, 116, 51, 67, 117, 114])
QGVUNX = "".join(chr(c) for c in [65, 109, 80, 109])
QJYMOU = "".join(chr(c) for c in [80, 117, 114, 103, 101, 83, 112, 101, 101, 100])
QLAIID = 90
QLNMHX = "".join(chr(c) for c in [70, 50, 50])
QNRSJM = 42
QNRXCH = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 49]
)
QPLSPF = "".join(chr(c) for c in [83, 101, 116, 112, 111, 105, 110, 116, 71])
QRJJJV = "".join(chr(c) for c in [70, 105, 108, 116, 70, 114, 101, 113])
QSNQLN = "".join(chr(c) for c in [70, 50])
QXPICX = "".join(chr(c) for c in [79, 117, 116, 52])
RAHEOC = 39
RJHIUS = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116])
RJJJVY = 3
RSJMCB = 43
RTFMNH = 8
RXCHWD = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 50]
)
RYXBQF = "".join(
    chr(c) for c in [77, 97, 120, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
SAKQXP = 13
SBDJQR = "".join(chr(c) for c in [70, 50, 50, 76, 105, 110, 101])
SELHBQ = 5
SIFJBI = "".join(chr(c) for c in [72, 101, 97, 116, 101, 114, 80, 117, 109, 112])
SIRYXB = "".join(
    chr(c) for c in [77, 105, 110, 83, 101, 116, 112, 111, 105, 110, 116, 71]
)
SJMCBF = "".join(chr(c) for c in [79, 117, 116, 57, 67, 117, 114])
SJWMNZ = 61
SKSOKP = "".join(chr(c) for c in [79, 117, 116, 56])
SKWIVD = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114])
SNQLNM = "".join(chr(c) for c in [70, 51])
SOKPHU = "".join(chr(c) for c in [79, 117, 116, 57])
SOOQNR = "".join(chr(c) for c in [79, 117, 116, 54, 67, 117, 114])
SPBWJY = 29
SPFTSI = 33
SSAKQX = "".join(chr(c) for c in [79, 117, 116, 50])
SSUHBV = "".join(chr(c) for c in [70, 50, 50, 67, 117, 114, 114, 101, 110, 116])
SUHBVW = 102
SXUJUT = "".join(chr(c) for c in [76, 73])
TFMNHT = "".join(chr(c) for c in [69, 99, 111, 110, 68, 117, 114])
THBSKS = 17
THECVY = "".join(chr(c) for c in [80, 50, 76])
TIACQF = 95
TIDUBS = 99
TOPHUG = "".join(
    chr(c)
    for c in [83, 111, 97, 107, 79, 110, 67, 117, 115, 116, 111, 109, 75, 101, 121]
)
TTIDUB = "".join(chr(c) for c in [70, 50, 67, 117, 114, 114, 101, 110, 116])
TYEKCW = 54
TYIYWS = "".join(
    chr(c) for c in [78, 111, 66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67]
)
UBSSUH = "".join(chr(c) for c in [70, 50, 49, 67, 117, 114, 114, 101, 110, 116])
UBYGDS = 105
UGSELH = "".join(chr(c) for c in [76, 97, 115, 116, 80, 117, 109, 112, 75, 101, 121])
UGTYIY = "".join(
    chr(c) for c in [77, 117, 108, 116, 105, 75, 101, 121, 79, 112, 116, 105, 111, 110]
)
UHBVWV = "".join(chr(c) for c in [70, 50, 51, 67, 117, 114, 114, 101, 110, 116])
UIKFWR = 114
UJUTYE = "".join(chr(c) for c in [76, 105, 103, 104, 116, 73, 110, 116, 115])
UNBLKX = 58
UNXNKM = "".join(
    chr(c)
    for c in [65, 109, 98, 105, 97, 110, 116, 79, 72, 84, 114, 105, 103, 65, 68, 67]
)
UOJRJH = 22
UQEXLS = 48
USOOQN = 25
USPBWJ = "".join(chr(c) for c in [79, 51, 80, 117, 109, 112])
UTOPHU = 72
UTYEKC = "".join(chr(c) for c in [80, 117, 109, 112, 84, 105, 109, 101, 79, 117, 116])
UXFEFJ = "".join(
    chr(c) for c in [70, 105, 108, 116, 73, 110, 116, 101, 114, 102, 97, 99, 101]
)
UYNQJY = "".join(
    chr(c) for c in [65, 108, 119, 97, 121, 115, 69, 110, 97, 98, 108, 101, 100]
)
VDNQGV = 75
VDQLAI = 89
VHFTHE = "".join(chr(c) for c in [80, 49, 72])
VKZILX = "".join(chr(c) for c in [79, 117, 116, 51, 70, 117, 115, 101])
VLASSA = "".join(chr(c) for c in [])
VUBYGD = "".join(chr(c) for c in [70, 50, 76, 105, 110, 101])
VWVUBY = 104
VYFCRT = 6
VYYPIP = "".join(chr(c) for c in [80, 52, 76])
WAJVDQ = "".join(chr(c) for c in [79, 117, 116, 54, 70, 117, 115, 101])
WAONPY = "".join(chr(c) for c in [76, 49, 50, 48, 84, 105, 109, 101, 79, 117, 116])
WBSIRY = "".join(
    chr(c) for c in [67, 111, 111, 108, 100, 111, 119, 110, 84, 105, 109, 101]
)
WDAFIK = "".join(
    chr(c) for c in [75, 101, 121, 112, 97, 100, 79, 112, 116, 105, 111, 110, 115, 52]
)
WJYKLG = 30
WMNZMJ = 62
WRKINE = "".join(chr(c) for c in [67, 69])
WSKWIV = 73
XBQFYL = "".join(chr(c) for c in [69, 99, 111, 110, 84, 121, 112, 101])
XCHWDA = 111
XFEFJT = 32
XLSXUJ = "".join(chr(c) for c in [79, 117, 116, 76, 105])
XNKMLO = "".join(
    chr(c)
    for c in [80, 117, 109, 112, 49, 85, 115, 101, 114, 65, 99, 99, 101, 115, 115]
)
XPICXQ = 15
XQGLRA = 37
XQIEFX = 26
XSJWMN = "".join(
    chr(c)
    for c in [
        70,
        105,
        108,
        116,
        79,
        110,
        84,
        105,
        109,
        101,
        68,
        117,
        114,
        105,
        110,
        103,
        79,
        84,
    ]
)
XTIACQ = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 49, 70, 117, 115, 101])
XWAJVD = 87
XYBQSN = "".join(chr(c) for c in [79, 117, 116, 49, 70, 117, 115, 101])
YBQSNQ = 83
YEKCWA = "".join(chr(c) for c in [80, 49, 76, 84, 105, 109, 101, 79, 117, 116])
YFCRTF = "".join(chr(c) for c in [70, 105, 108, 116, 68, 117, 114, 50])
YGDSBD = 106
YIYWSK = "".join(chr(c) for c in [66, 108, 111, 119, 101, 114, 79, 110, 73, 50, 67])
YKLGQP = "".join(chr(c) for c in [84, 111, 103, 103, 108, 101])
YLJUIK = "".join(
    chr(c) for c in [78, 111, 72, 101, 97, 116, 80, 101, 114, 105, 111, 100]
)
YMOUNB = "".join(chr(c) for c in [72, 105])
YNQJYM = "".join(
    chr(c) for c in [87, 105, 116, 104, 83, 80, 79, 118, 101, 114, 57, 53, 70]
)
YOUSPB = "".join(chr(c) for c in [65, 108, 119, 97, 121, 115])
YPIPIV = "".join(chr(c) for c in [66, 76, 79])
YWSKWI = "".join(chr(c) for c in [77, 97, 115, 116, 101, 114, 83, 108, 97, 118, 101])
YXBQFY = 68
YYLIUX = "".join(chr(c) for c in [79, 119, 110])
YYPIPI = "".join(chr(c) for c in [80, 53])
ZCQBMJ = "".join(
    chr(c) for c in [67, 111, 110, 102, 105, 103, 78, 117, 109, 98, 101, 114]
)
ZILXWA = "".join(chr(c) for c in [79, 117, 116, 52, 70, 117, 115, 101])
ZMJIGY = "".join(chr(c) for c in [80, 49])
ZUQEXL = "".join(chr(c) for c in [68, 105, 114, 101, 99, 116, 67, 117, 114])
AFIKJP = [
    BMJVHF,
    SSAKQX,
    AKQXPI,
    QXPICX,
    PICXQI,
    CXQIEF,
    CTHBSK,
    HBSKSO,
    SKSOKP,
    SOKPHU,
    KPHUOJ,
    HUOJRJ,
    OJRJHI,
    RJHIUS,
    IUSOOQ,
    XLSXUJ,
]
ASSAKQ = [
    JVHFTH,
    VHFTHE,
    HFTHEC,
    FTHECV,
    THECVY,
    HECVYY,
    ECVYYP,
    CVYYPI,
    VYYPIP,
    YYPIPI,
    YPIPIV,
    PIPIVL,
    IPIVLA,
    PIVLAS,
    IVLASS,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    LASSAK,
]
BXYBQS = [JYKLGQ, IBXYBQ]
FIKJPU = [SXUJUT]
FYLJUI = [JYKLGQ, QFYLJU]
GSELHB = [JUGSEL, UGSELH]
HIUSOO = [
    JVHFTH,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    PIPIVL,
]
IKJPUN = []
IYWSKW = [TYIYWS, YIYWSK]
KLGQPL = [JYKLGQ, YKLGQP]
MJIGYO = [JVHFTH, ZMJIGY, PIPIVL]
MJMAOA = [BIAMJM, IAMJMA, AMJMAO]
MLOIJU = [NKMLOI, KMLOIJ]
MOUNBL = [JYMOUN, YMOUNB]
NQJYMO = [OUYNQJ, UYNQJY, YNQJYM]
OAWBSI = [MAOAWB, AOAWBS]
OUSPBW = [GYOUSP, YOUSPB]
PBWJYK = [PIPIVL, ZMJIGY]
QIEFXQ = [
    JVHFTH,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    IVLASS,
]
QIPOUY = [CPQIPO, PQIPOU]
RKINEJ = [FWRKIN, WRKINE]
TACCPQ = [FEFJTA, EFJTAC, FJTACC, JTACCP]
TBJEUT = [JVHFTH, CPQIPO, HTBJEU]
TSIFJB = [PFTSIF, FTSIFJ]
VUNXNK = [JVHFTH, QGVUNX, GVUNXN]
WIVDNQ = [VLASSA, SKWIVD, KWIVDN]
WVUBYG = [VLASSA, VLASSA, VLASSA, VLASSA, VLASSA, VLASSA, NMHXEK, MHXEKV, HXEKVK]
XEKVKZ = [BQSNQL, QSNQLN, SNQLNM, NQLNMH, QLNMHX, LNMHXE, NMHXEK, MHXEKV, HXEKVK]
XUJUTY = [
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    VLASSA,
    SXUJUT,
]
YLIUXF = [PYYLIU, YYLIUX]


class GeckoConfigStruct:
    def __init__(self, struct_):
        self.struct = struct_

    @property
    def version(self):
        return KJPUNR

    @property
    def output_keys(self):
        return AFIKJP

    @property
    def accessors(self):
        return {
            ZCQBMJ: GeckoByteStructAccessor(self.struct, ZCQBMJ, CQBMJV, QBMJVH),
            BMJVHF: GeckoEnumStructAccessor(
                self.struct, BMJVHF, MJVHFT, None, ASSAKQ, None, None, QBMJVH
            ),
            SSAKQX: GeckoEnumStructAccessor(
                self.struct, SSAKQX, SAKQXP, None, ASSAKQ, None, None, QBMJVH
            ),
            AKQXPI: GeckoEnumStructAccessor(
                self.struct, AKQXPI, KQXPIC, None, ASSAKQ, None, None, QBMJVH
            ),
            QXPICX: GeckoEnumStructAccessor(
                self.struct, QXPICX, XPICXQ, None, ASSAKQ, None, None, QBMJVH
            ),
            PICXQI: GeckoEnumStructAccessor(
                self.struct, PICXQI, ICXQIE, None, ASSAKQ, None, None, QBMJVH
            ),
            CXQIEF: GeckoEnumStructAccessor(
                self.struct, CXQIEF, XQIEFX, None, QIEFXQ, None, None, QBMJVH
            ),
            IEFXQG: GeckoByteStructAccessor(self.struct, IEFXQG, EFXQGL, QBMJVH),
            FXQGLR: GeckoByteStructAccessor(self.struct, FXQGLR, XQGLRA, QBMJVH),
            QGLRAH: GeckoByteStructAccessor(self.struct, QGLRAH, GLRAHE, QBMJVH),
            LRAHEO: GeckoByteStructAccessor(self.struct, LRAHEO, RAHEOC, QBMJVH),
            AHEOCT: GeckoByteStructAccessor(self.struct, AHEOCT, HEOCTH, QBMJVH),
            EOCTHB: GeckoByteStructAccessor(self.struct, EOCTHB, OCTHBS, QBMJVH),
            CTHBSK: GeckoEnumStructAccessor(
                self.struct, CTHBSK, THBSKS, None, ASSAKQ, None, None, QBMJVH
            ),
            HBSKSO: GeckoEnumStructAccessor(
                self.struct, HBSKSO, BSKSOK, None, ASSAKQ, None, None, QBMJVH
            ),
            SKSOKP: GeckoEnumStructAccessor(
                self.struct, SKSOKP, KSOKPH, None, ASSAKQ, None, None, QBMJVH
            ),
            SOKPHU: GeckoEnumStructAccessor(
                self.struct, SOKPHU, OKPHUO, None, ASSAKQ, None, None, QBMJVH
            ),
            KPHUOJ: GeckoEnumStructAccessor(
                self.struct, KPHUOJ, PHUOJR, None, ASSAKQ, None, None, QBMJVH
            ),
            HUOJRJ: GeckoEnumStructAccessor(
                self.struct, HUOJRJ, UOJRJH, None, ASSAKQ, None, None, QBMJVH
            ),
            OJRJHI: GeckoEnumStructAccessor(
                self.struct, OJRJHI, JRJHIU, None, ASSAKQ, None, None, QBMJVH
            ),
            RJHIUS: GeckoEnumStructAccessor(
                self.struct, RJHIUS, JHIUSO, None, HIUSOO, None, None, QBMJVH
            ),
            IUSOOQ: GeckoEnumStructAccessor(
                self.struct, IUSOOQ, USOOQN, None, HIUSOO, None, None, QBMJVH
            ),
            SOOQNR: GeckoByteStructAccessor(self.struct, SOOQNR, OOQNRS, QBMJVH),
            OQNRSJ: GeckoByteStructAccessor(self.struct, OQNRSJ, QNRSJM, QBMJVH),
            NRSJMC: GeckoByteStructAccessor(self.struct, NRSJMC, RSJMCB, QBMJVH),
            SJMCBF: GeckoByteStructAccessor(self.struct, SJMCBF, JMCBFE, QBMJVH),
            MCBFEG: GeckoByteStructAccessor(self.struct, MCBFEG, CBFEGZ, QBMJVH),
            BFEGZU: GeckoByteStructAccessor(self.struct, BFEGZU, FEGZUQ, QBMJVH),
            EGZUQE: GeckoByteStructAccessor(self.struct, EGZUQE, GZUQEX, QBMJVH),
            ZUQEXL: GeckoByteStructAccessor(self.struct, ZUQEXL, UQEXLS, QBMJVH),
            QEXLSX: GeckoByteStructAccessor(self.struct, QEXLSX, EXLSXU, QBMJVH),
            XLSXUJ: GeckoEnumStructAccessor(
                self.struct, XLSXUJ, LSXUJU, None, XUJUTY, None, None, None
            ),
            UJUTYE: GeckoByteStructAccessor(self.struct, UJUTYE, JUTYEK, None),
            UTYEKC: GeckoByteStructAccessor(self.struct, UTYEKC, TYEKCW, QBMJVH),
            YEKCWA: GeckoByteStructAccessor(self.struct, YEKCWA, EKCWAO, QBMJVH),
            KCWAON: GeckoByteStructAccessor(self.struct, KCWAON, CWAONP, QBMJVH),
            WAONPY: GeckoByteStructAccessor(self.struct, WAONPY, AONPYY, QBMJVH),
            ONPYYL: GeckoEnumStructAccessor(
                self.struct, ONPYYL, NPYYLI, LIUXFE, YLIUXF, None, IUXFEF, QBMJVH
            ),
            UXFEFJ: GeckoEnumStructAccessor(
                self.struct, UXFEFJ, XFEFJT, None, TACCPQ, None, None, QBMJVH
            ),
            ACCPQI: GeckoEnumStructAccessor(
                self.struct, ACCPQI, CCPQIP, None, QIPOUY, None, None, QBMJVH
            ),
            IPOUYN: GeckoEnumStructAccessor(
                self.struct, IPOUYN, POUYNQ, None, NQJYMO, None, None, QBMJVH
            ),
            QJYMOU: GeckoEnumStructAccessor(
                self.struct, QJYMOU, NPYYLI, IUXFEF, MOUNBL, None, IUXFEF, QBMJVH
            ),
            OUNBLK: GeckoByteStructAccessor(self.struct, OUNBLK, UNBLKX, QBMJVH),
            NBLKXS: GeckoByteStructAccessor(self.struct, NBLKXS, BLKXSJ, QBMJVH),
            LKXSJW: GeckoByteStructAccessor(self.struct, LKXSJW, KXSJWM, QBMJVH),
            XSJWMN: GeckoByteStructAccessor(self.struct, XSJWMN, SJWMNZ, QBMJVH),
            JWMNZM: GeckoByteStructAccessor(self.struct, JWMNZM, WMNZMJ, QBMJVH),
            MNZMJI: GeckoEnumStructAccessor(
                self.struct, MNZMJI, NZMJIG, None, MJIGYO, None, None, QBMJVH
            ),
            JIGYOU: GeckoEnumStructAccessor(
                self.struct, JIGYOU, IGYOUS, None, OUSPBW, None, None, QBMJVH
            ),
            USPBWJ: GeckoEnumStructAccessor(
                self.struct, USPBWJ, SPBWJY, None, PBWJYK, None, None, QBMJVH
            ),
            BWJYKL: GeckoEnumStructAccessor(
                self.struct, BWJYKL, WJYKLG, None, KLGQPL, None, None, QBMJVH
            ),
            LGQPLS: GeckoByteStructAccessor(self.struct, LGQPLS, GQPLSP, QBMJVH),
            QPLSPF: GeckoTempStructAccessor(self.struct, QPLSPF, PLSPFT, QBMJVH),
            LSPFTS: GeckoEnumStructAccessor(
                self.struct, LSPFTS, SPFTSI, None, TSIFJB, None, None, QBMJVH
            ),
            SIFJBI: GeckoEnumStructAccessor(
                self.struct, SIFJBI, IFJBIA, None, PBWJYK, None, None, QBMJVH
            ),
            FJBIAM: GeckoEnumStructAccessor(
                self.struct, FJBIAM, JBIAMJ, None, MJMAOA, None, None, QBMJVH
            ),
            JMAOAW: GeckoEnumStructAccessor(
                self.struct, JMAOAW, NPYYLI, AWBSIR, OAWBSI, None, IUXFEF, QBMJVH
            ),
            WBSIRY: GeckoByteStructAccessor(self.struct, WBSIRY, BSIRYX, QBMJVH),
            SIRYXB: GeckoTempStructAccessor(self.struct, SIRYXB, IRYXBQ, QBMJVH),
            RYXBQF: GeckoTempStructAccessor(self.struct, RYXBQF, YXBQFY, QBMJVH),
            XBQFYL: GeckoEnumStructAccessor(
                self.struct, XBQFYL, BQFYLJ, None, FYLJUI, None, None, QBMJVH
            ),
            YLJUIK: GeckoByteStructAccessor(self.struct, YLJUIK, LJUIKF, QBMJVH),
            JUIKFW: GeckoByteStructAccessor(self.struct, JUIKFW, UIKFWR, QBMJVH),
            IKFWRK: GeckoEnumStructAccessor(
                self.struct, IKFWRK, KFWRKI, None, RKINEJ, None, None, QBMJVH
            ),
            KINEJN: GeckoByteStructAccessor(self.struct, KINEJN, INEJNI, QBMJVH),
            NEJNIB: GeckoByteStructAccessor(self.struct, NEJNIB, EJNIBX, QBMJVH),
            JNIBXY: GeckoEnumStructAccessor(
                self.struct, JNIBXY, NIBXYB, None, BXYBQS, None, None, QBMJVH
            ),
            XYBQSN: GeckoEnumStructAccessor(
                self.struct, XYBQSN, YBQSNQ, None, XEKVKZ, None, None, QBMJVH
            ),
            EKVKZI: GeckoEnumStructAccessor(
                self.struct, EKVKZI, KVKZIL, None, XEKVKZ, None, None, QBMJVH
            ),
            VKZILX: GeckoEnumStructAccessor(
                self.struct, VKZILX, KZILXW, None, XEKVKZ, None, None, QBMJVH
            ),
            ZILXWA: GeckoEnumStructAccessor(
                self.struct, ZILXWA, ILXWAJ, None, XEKVKZ, None, None, QBMJVH
            ),
            LXWAJV: GeckoEnumStructAccessor(
                self.struct, LXWAJV, XWAJVD, None, XEKVKZ, None, None, QBMJVH
            ),
            WAJVDQ: GeckoEnumStructAccessor(
                self.struct, WAJVDQ, AJVDQL, None, XEKVKZ, None, None, QBMJVH
            ),
            JVDQLA: GeckoEnumStructAccessor(
                self.struct, JVDQLA, VDQLAI, None, XEKVKZ, None, None, QBMJVH
            ),
            DQLAII: GeckoEnumStructAccessor(
                self.struct, DQLAII, QLAIID, None, XEKVKZ, None, None, QBMJVH
            ),
            LAIIDN: GeckoEnumStructAccessor(
                self.struct, LAIIDN, AIIDNI, None, XEKVKZ, None, None, QBMJVH
            ),
            IIDNIB: GeckoEnumStructAccessor(
                self.struct, IIDNIB, IDNIBX, None, XEKVKZ, None, None, QBMJVH
            ),
            DNIBXT: GeckoEnumStructAccessor(
                self.struct, DNIBXT, NIBXTI, None, XEKVKZ, None, None, QBMJVH
            ),
            IBXTIA: GeckoEnumStructAccessor(
                self.struct, IBXTIA, BXTIAC, None, XEKVKZ, None, None, QBMJVH
            ),
            XTIACQ: GeckoEnumStructAccessor(
                self.struct, XTIACQ, TIACQF, None, XEKVKZ, None, None, QBMJVH
            ),
            IACQFF: GeckoEnumStructAccessor(
                self.struct, IACQFF, ACQFFT, None, XEKVKZ, None, None, QBMJVH
            ),
            CQFFTT: GeckoEnumStructAccessor(
                self.struct, CQFFTT, QFFTTI, None, XEKVKZ, None, None, QBMJVH
            ),
            FFTTID: GeckoByteStructAccessor(self.struct, FFTTID, FTTIDU, QBMJVH),
            TTIDUB: GeckoByteStructAccessor(self.struct, TTIDUB, TIDUBS, QBMJVH),
            IDUBSS: GeckoByteStructAccessor(self.struct, IDUBSS, DUBSSU, QBMJVH),
            UBSSUH: GeckoByteStructAccessor(self.struct, UBSSUH, BSSUHB, QBMJVH),
            SSUHBV: GeckoByteStructAccessor(self.struct, SSUHBV, SUHBVW, QBMJVH),
            UHBVWV: GeckoByteStructAccessor(self.struct, UHBVWV, HBVWVU, QBMJVH),
            BVWVUB: GeckoEnumStructAccessor(
                self.struct, BVWVUB, VWVUBY, None, WVUBYG, None, None, QBMJVH
            ),
            VUBYGD: GeckoEnumStructAccessor(
                self.struct, VUBYGD, UBYGDS, None, WVUBYG, None, None, QBMJVH
            ),
            BYGDSB: GeckoEnumStructAccessor(
                self.struct, BYGDSB, YGDSBD, None, WVUBYG, None, None, QBMJVH
            ),
            GDSBDJ: GeckoEnumStructAccessor(
                self.struct, GDSBDJ, DSBDJQ, None, WVUBYG, None, None, QBMJVH
            ),
            SBDJQR: GeckoEnumStructAccessor(
                self.struct, SBDJQR, BDJQRJ, None, WVUBYG, None, None, QBMJVH
            ),
            DJQRJJ: GeckoEnumStructAccessor(
                self.struct, DJQRJJ, JQRJJJ, None, WVUBYG, None, None, QBMJVH
            ),
            QRJJJV: GeckoByteStructAccessor(self.struct, QRJJJV, RJJJVY, QBMJVH),
            JJJVYF: GeckoTimeStructAccessor(self.struct, JJJVYF, JJVYFC, QBMJVH),
            JVYFCR: GeckoTimeStructAccessor(self.struct, JVYFCR, VYFCRT, QBMJVH),
            YFCRTF: GeckoTimeStructAccessor(self.struct, YFCRTF, FCRTFM, QBMJVH),
            CRTFMN: GeckoTimeStructAccessor(self.struct, CRTFMN, RTFMNH, QBMJVH),
            TFMNHT: GeckoTimeStructAccessor(self.struct, TFMNHT, FMNHTB, QBMJVH),
            MNHTBJ: GeckoEnumStructAccessor(
                self.struct, MNHTBJ, NHTBJE, None, TBJEUT, None, None, QBMJVH
            ),
            BJEUTO: GeckoBoolStructAccessor(
                self.struct, BJEUTO, JEUTOP, LIUXFE, QBMJVH
            ),
            EUTOPH: GeckoBoolStructAccessor(
                self.struct, EUTOPH, UTOPHU, IUXFEF, QBMJVH
            ),
            TOPHUG: GeckoBoolStructAccessor(
                self.struct, TOPHUG, UTOPHU, LIUXFE, QBMJVH
            ),
            OPHUGT: GeckoBoolStructAccessor(
                self.struct, OPHUGT, UTOPHU, PHUGTY, QBMJVH
            ),
            HUGTYI: GeckoBoolStructAccessor(
                self.struct, HUGTYI, UTOPHU, AWBSIR, QBMJVH
            ),
            UGTYIY: GeckoEnumStructAccessor(
                self.struct, UGTYIY, GTYIYW, None, IYWSKW, None, None, QBMJVH
            ),
            YWSKWI: GeckoEnumStructAccessor(
                self.struct, YWSKWI, WSKWIV, None, WIVDNQ, None, None, QBMJVH
            ),
            IVDNQG: GeckoByteStructAccessor(self.struct, IVDNQG, VDNQGV, QBMJVH),
            DNQGVU: GeckoEnumStructAccessor(
                self.struct, DNQGVU, NQGVUN, None, VUNXNK, None, None, QBMJVH
            ),
            UNXNKM: GeckoWordStructAccessor(self.struct, UNXNKM, NXNKML, QBMJVH),
            XNKMLO: GeckoEnumStructAccessor(
                self.struct, XNKMLO, NPYYLI, PHUGTY, MLOIJU, None, IUXFEF, QBMJVH
            ),
            LOIJUG: GeckoBoolStructAccessor(
                self.struct, LOIJUG, NPYYLI, OIJUGS, QBMJVH
            ),
            IJUGSE: GeckoEnumStructAccessor(
                self.struct, IJUGSE, NPYYLI, SELHBQ, GSELHB, None, IUXFEF, QBMJVH
            ),
            ELHBQN: GeckoBoolStructAccessor(
                self.struct, ELHBQN, NPYYLI, LHBQNR, QBMJVH
            ),
            HBQNRX: GeckoBoolStructAccessor(
                self.struct, HBQNRX, NPYYLI, BQNRXC, QBMJVH
            ),
            QNRXCH: GeckoByteStructAccessor(self.struct, QNRXCH, NRXCHW, QBMJVH),
            RXCHWD: GeckoByteStructAccessor(self.struct, RXCHWD, XCHWDA, QBMJVH),
            CHWDAF: GeckoByteStructAccessor(self.struct, CHWDAF, HWDAFI, QBMJVH),
            WDAFIK: GeckoByteStructAccessor(self.struct, WDAFIK, DAFIKJ, QBMJVH),
        }



""" Spa Structure block """

import logging

from .protocol import GeckoStatusBlockProtocolHandler
from .udp_socket import GeckoUdpSocket

logger = logging.getLogger(__name__)


class GeckoStructure:
    """Class to host/manage the raw data block for a spa structure"""

    def __init__(self, on_set_value):
        self._status_block = b"\x00" * 1024
        self.accessors = {}
        self.all_outputs = []
        self.all_devices = []
        self.user_demands = []
        self.had_at_least_one_block = False
        self._on_set_value = on_set_value

        # Install a set of handlers

    def replace_status_block_segment(self, offset, segment):
        """Replace a segment of the status block"""
        previous_block = self._status_block
        segment_len = len(segment)
        self._status_block = (
            self._status_block[0:offset]
            + segment
            + self._status_block[offset + segment_len :]
        )
        # Notify changes to accessors
        for accessor in self.accessors.values():
            accessor.status_block_changed(offset, segment_len, previous_block)

    @property
    def status_block(self):
        return self._status_block

    def set_status_block(self, block):
        self._status_block = block

    def _on_status_block_received(
        self, handler: GeckoStatusBlockProtocolHandler, sender
    ):
        if not self._next_expected == handler.sequence:
            logger.debug(
                "Out-of-sequence status block segment %d - ignored", handler.sequence
            )
            if handler.next == 0:
                logger.debug("Retry status block request")
                self._next_expected = 0
                self._status_block_segments = []
                if not handler.retry(self._socket):
                    raise RuntimeError("Too many retries")
        else:
            self._status_block_segments.append(handler.data)
            self._next_expected = handler.next
            # When we get the last partial segment, we can assume the spa is
            # connected and we can report on status
            if handler.next == 0:
                logger.debug(
                    "Status block segments complete, update and remove handler"
                )
                self.replace_status_block_segment(
                    self._status_block_offset, b"".join(self._status_block_segments)
                )
                self.had_at_least_one_block = True
                handler._should_remove_handler = True

    def build_accessors(self, config_class, log_class):
        self.accessors = dict(config_class.accessors, **log_class.accessors)
        # Get all outputs
        self.all_outputs = config_class.output_keys
        # Get collection of possible devices
        self.all_devices = log_class.all_device_keys
        # User devices are those that have a Ud in the tag name
        self.user_demands = log_class.user_demand_keys

    def retry_request(
        self,
        socket_: GeckoUdpSocket,
        request: GeckoStatusBlockProtocolHandler,
        sender: tuple,
    ):
        self._socket = socket_
        request._on_handled = self._on_status_block_received
        self._status_block_offset = request.start
        socket_.add_receive_handler(request)
        self._next_expected = 0
        self._status_block_segments = []
        socket_.queue_send(request, sender)

    def set_value(self, pos, length, newvalue):
        # Delegate this
        self._on_set_value(pos, length, newvalue)



""" Shared command functionality """

import cmd
import logging

LICENSE = """
#
#   Copyright (C) 2020, Gazoodle (https://github.com/gazoodle)
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
"""


class GeckoCmd(cmd.Cmd):
    """GeckoCmd is a shared command processor. It implements shared
    functionality between the Shell and the Simulator"""

    BANNER = None

    @classmethod
    def run(cls, first_commands=None):
        """Convenience function to run a command loop"""
        with cls(first_commands) as cmd:
            cmd.cmdloop()

    def __init__(self, first_commands=None):
        super().__init__()

        if self.BANNER is not None:
            print(self.BANNER)

        self.stream_logger = None
        self.file_logger = None
        self._init_logging()

        if first_commands is not None:
            for command in first_commands:
                self.onecmd(command)

    def __enter__(self):
        return self

    def __exit__(self, *args):
        pass

    def do_exit(self, _arg):
        """Exit this shell: exit"""
        return True

    def do_loglevel(self, arg):
        """Set the logging level : loglevel <ERROR|WARNING|INFO|DEBUG>"""
        for handler in logging.getLogger().handlers:
            handler.setLevel(arg)
        self._set_root_log_level()

    def do_logfile(self, arg):
        """Add a file logger to the system : logfile <filename>"""
        if self.file_logger is not None:
            print("There is already a file logger installed")
            return
        self.file_logger = logging.FileHandler(arg)
        self.file_logger.setLevel(logging.DEBUG)
        self.file_logger.setFormatter(
            logging.Formatter("%(asctime)s %(name)s %(levelname)s %(message)s")
        )
        logging.getLogger().addHandler(self.file_logger)
        self._set_root_log_level()

    def _init_logging(self):
        self.stream_logger = logging.StreamHandler()
        self.stream_logger.setLevel(logging.WARNING)
        self.stream_logger.setFormatter(
            logging.Formatter("LOG> %(levelname)s %(message)s")
        )
        logging.getLogger().addHandler(self.stream_logger)
        self._set_root_log_level()

    def _set_root_log_level(self):
        # Set root log level
        logging.getLogger().setLevel(
            min(handler.level for handler in logging.getLogger().handlers)
        )

    def do_license(self, _arg):
        """Display the license details : license"""
        print(LICENSE)
